/* Controller.java
 * CISC475 Project
 * Group 02
 */

package tlass.project;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import tlass.project.gui.*;
import tlass.project.model.Assistant;
import tlass.project.model.Constraint;
import tlass.project.model.Instructor;
import tlass.project.model.Model;
import tlass.project.model.Position;
import tlass.project.model.Response;
import tlass.project.model.Schedule;
import tlass.project.model.Section;
import tlass.project.model.Skill;

import au.com.bytecode.opencsv.CSVReader;


public class Controller {
	
	//variables for new system design
	private LoginHandler loginHandler;
	private SemesterHandler semesterHandler;
	private WebformHandler webformHandler;
	private Scheduler scheduler;
	private User user;
	private String semester;
	private Model model;
	private View view;
	private ModelHandler modelHandler;
	private CSVReader reader;
	
	/**
	 * Main calling function. Control moves to either graphical or command line from here.
	 */
	public static void main(String[] args) {
		Controller controller = new Controller();
		controller.runGraphicalView();
	}
	
	/**
	 * Constructor to give instances to important class members
	 */
	public Controller() {
		model= new Model();
		model.createDummyData();
		try {
			reader = new CSVReader(new FileReader("settings"));
			String [] nextLine;
		   	if((nextLine = reader.readNext()) != null) {
		   		DataStoreConnection.host = nextLine[0];
				DataStoreConnection.user = nextLine[1];
				DataStoreConnection.pass = nextLine[2];
				DataStoreConnection.db = nextLine[3];
		    }
		    reader.close();
		} catch (IOException e) {
			System.out.println("Cannot read settings file");
		}
		
		user= new User();
		modelHandler= new ModelHandler(model);
		loginHandler= new LoginHandler();
		semesterHandler= new SemesterHandler();
		webformHandler= new WebformHandler(this);
	}

	

	/**
	 * Establish graphical View, and begin login process
	 */
	public void runGraphicalView() {
		view = new View(this);
		view.openLoginWindow();
	}
	
	// Getters
	/**
	 * Returns the current view
	 */
	public View getView(){ return view; }
	
	/**
	 * Returns the constraints currently applied to the model
	 * @return
	 */
	public ArrayList<Constraint> getConstraints(){ return model.getConstraints(); }
	
	/**
	 * Returns the assistants currently in the model.
	 * @return
	 */
	public ArrayList<Assistant> getAssistants(){ return model.getAssistants(); }
	
	/**
	 * Returns the sections currently in the model.
	 * @return
	 */
	public ArrayList<Section> getSections(){ return model.getSections(); }
	
	/**
	 * Returns the schedules currently in the model.
	 * @return
	 */
	public ArrayList<Schedule> getSchedules() {return model.getSchedules();}
	
	// Setters
	/**
	 * Sets the current view to the one supplied
	 */
	public void setView(View v){ view = v; }	
	
	/**
	 * Sets the current constraints to the ones supplied.
	 * @param c
	 */
	public void setConstraints(ArrayList<Constraint> c){ model.setConstraints(c); }
	
	/**
	 * Set the Assistants in the model to the ones specified.
	 * @param a
	 */
	public void setAssistants(ArrayList<Assistant> a){ model.setAssistants(a); }
	
	/**
	 * Set the sections in the model to the ones specified.
	 * @param s
	 */
	public void setSections(ArrayList<Section> s){ model.setSections(s); }
	
	/**
	 * Set the schedules in the model to the ones specified.
	 * @param s
	 */
	public void setSchedules(ArrayList<Schedule> s) { model.setSchedules(s); }

	/**
	 * Sets the current user to the one specified.
	 * @param user
	 */
	public void setUser(User user) {
		this.user = user;
	}

	/**
	 * Returns the current user
	 * @return
	 */
	public User getUser() {
		return user;
	}

	/**
	 * Sets the current LoginHandler to the one supplied
	 * @param loginHandler
	 */
	public void setLoginHandler(LoginHandler loginHandler) {
		this.loginHandler = loginHandler;
	}

	/**
	 * Returns the current LoginHanlder
	 * @return
	 */
	public LoginHandler getLoginHandler() {
		return loginHandler;
	}

	/**
	 * Sets the current semesterHandler to the one supplied
	 * @param semesterHandler
	 */
	public void setSemesterHandler(SemesterHandler semesterHandler) {
		this.semesterHandler = semesterHandler;
	}

	/**
	 * Returns the current semesterHandler
	 * @return
	 */
	public SemesterHandler getSemesterHandler() {
		return semesterHandler;
	}

	/**
	 * Sets the current webformHandler to the one supplied.
	 * @param webformHandler
	 */
	public void setWebformHandler(WebformHandler webformHandler) {
		this.webformHandler = webformHandler;
	}

	/**
	 * Returns the current webformHandler
	 * @return
	 */
	public WebformHandler getWebformHandler() {
		return webformHandler;
	}

	/**
	 * Returns the current modelHanlder
	 * @return
	 */
	public ModelHandler getModelHandler() {
		return modelHandler;
	}
	
	/**
	 * Retrieves the current model
	 * @return The current model
	 */
	public Model getModel() {
		return model;
	}

	/**
	 * Sets the current model
	 * @param newModel
	 */
	public void setModel(Model newModel) {
		this.model= newModel;
	}

}
