package tlass.project;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.security.NoSuchAlgorithmException;
import java.sql.*;

import tlass.project.exceptions.DbConnectionException;


/**
 * Simplistic factory to interact with the Database
 * Generates a specialized Connection
 * 
 * @author acbart
 * 
 */
public class DataStoreConnection {
	private Connection connection;
	public static String host;
	public static String user;
	public static String pass;
	public static String db;

	public final static int MYSQL = 0;
	//public final static int POSTGRESQL = 1;
	//public final static int SQLITE = 2;

	/**
	 * Creates the MYSQL Connection 
	 */
	public void MySQLConnection(String host, String db, String user, String pass)
			throws DbConnectionException {
		try {
			Class.forName("com.mysql.jdbc.Driver");
			connection = DriverManager.getConnection("jdbc:mysql://" + host
					+ ":3306/" + db, user, pass);
			if (connection == null)
				throw new DbConnectionException("Connection Failed");
		} catch (SQLException e) {
			throw new DbConnectionException("Connection Failed");
		} catch (ClassNotFoundException e) {
			throw new DbConnectionException("No MySQL JDBC Driver can be found");
		}
	}

	/**
	 * Queries the DataStore and retrieves the results
	 */
	public ResultSet query(String query) throws SQLException {
		Statement statement = connection.createStatement();
		statement.executeQuery(query);
		return statement.getResultSet();
	}

	/**
	 * Close the connection to the DataStore
	 */
	public void close() {
		if (connection != null) {
			try {
				connection.close();
			} catch (Exception e) {
				System.out.println("Error Closing Connection");
				return;
			}
		}
	}

	/**
	 * Retrieves the connection
	 * @return
	 */
	public Connection get() {
		return connection;
	}

	public DataStoreConnection() throws DbConnectionException {
		// TODO: Encrypt any raw password strings for security
		connection = null;
		MySQLConnection(host, db, user, pass);
	}
	
	/**
	 * Tests the connection
	 * @param testHost
	 * @param testUser
	 * @param testPass
	 * @param testDb
	 * @throws DbConnectionException
	 */
	public DataStoreConnection(String testHost, String testUser, String testPass, String testDb) throws DbConnectionException {
		// TODO: Encrypt any raw password strings for security
		connection = null;
		MySQLConnection(testHost, testDb, testUser, testPass);
	}

	/**
	 * Writes out the data from the DataStore
	 * @param anObject
	 * @return
	 * @throws IOException
	 */
	public static byte[] writeBytes(Object anObject) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream out = new ObjectOutputStream(baos);
		out.writeObject(anObject);
		out.close();
		return baos.toByteArray();
	}

	/**
	 * Reads in the data from the DataStore
	 * @param byteObject
	 * @return
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public static Object readBytes(byte[] byteObject) throws IOException,
			ClassNotFoundException {
		ByteArrayInputStream bais = new ByteArrayInputStream(byteObject);
		ObjectInputStream in = new ObjectInputStream(bais);
		Object anObject = in.readObject();
		in.close();
		return anObject;
	}

	/**
	 * Pattern taken from http://www.rgagnon.com/javadetails/java-0400.html
	 * TODO: Properly cite usage!
	 * Performs a SHA-1 encryption algorithm on a string
	 * @param aString
	 * @return
	 * @throws NoSuchAlgorithmException
	 */
	public static String encrypt(String aString)
			throws NoSuchAlgorithmException {
		java.security.MessageDigest digest = null;
		digest = java.security.MessageDigest.getInstance("SHA-1");
		digest.reset();
		digest.update(aString.getBytes());
		return byteArrayToHexString(digest.digest());
	}

	/**
	 * Function taken from http://www.rgagnon.com/javadetails/java-0400.html
	 * @param Consumes a byte array to a String
	 * @return A string representation of a byte array
	 */
	public static String byteArrayToHexString(byte[] b) {
		StringBuffer sb = new StringBuffer(b.length * 2);
		for (int i = 0; i < b.length; i++) {
			int v = b[i] & 0xff;
			if (v < 16) {
				sb.append('0');
			}
			sb.append(Integer.toHexString(v));
		}
		return sb.toString().toUpperCase();
	}
}
