package tlass.project.constraintTypes;

import java.util.HashMap;

import tlass.project.model.Assistant;
import tlass.project.model.Constraint;
import tlass.project.model.Position;
import tlass.project.model.Section;

import JaCoP.constraints.CountStudents;
import JaCoP.constraints.XlteqC;
import JaCoP.core.IntVar;
import JaCoP.core.Store;

public class LimitStudentTotalConstraint extends Constraint{

	/**
	 * Constructor for this constraint.
	 */
	public LimitStudentTotalConstraint() {
		name= "Limit Student Total";
		description= "";
	}

	/**
	 * Limit Student Count: When a student is being assigned to 
	 * multiple positions, limit the total number of students they 
	 * can teach overall.
	 */
	@Override
	public void imposeConstraint(Store jacopStore, IntVar[] variables,
			Position[] positions, Assistant[] assistants) {
		HashMap<IntVar, Integer> weights= new HashMap<IntVar, Integer>();
		for (int i= 0; i < variables.length; i+= 1) {
			Section aSection =positions[i].getSection();
			weights.put(variables[i], aSection.getEnrolledMax());
		}
		for (int j= 0; j < assistants.length; j+= 1) {
			Assistant anAssistant= assistants[j];
			int maxResponsible= anAssistant.getStudentsLimit();
			IntVar maxVariable= new IntVar(jacopStore, anAssistant+"responsibleFor",
					0, maxResponsible+1);
			jacopStore.impose(new XlteqC(maxVariable, maxResponsible));
			jacopStore.impose(new CountStudents(variables, maxVariable, j, weights));
		}
	}

}
