package tlass.project.constraintTypes;

import tlass.project.model.Assistant;
import tlass.project.model.Constraint;
import tlass.project.model.Position;
import tlass.project.model.Section;
import JaCoP.constraints.XneqC;
import JaCoP.core.IntVar;
import JaCoP.core.Store;

public class PreventScheduleConflictsConstraint extends Constraint{

	/**
	 * Constructor for this constraint.
	 */
	public PreventScheduleConflictsConstraint() {
		name= "Prevent Schedule Conflicts";
		description= "";
	}

	/**
	 * Avoid Unavailable Times: If an assistant is assigned 
	 * to a position, that positions time must not intersect
	 * with a time that assistant has declared to be unavailable.
	 */
	@Override
	public void imposeConstraint(Store jacopStore, IntVar[] variables,
			Position[] positions, Assistant[] assistants) {
		for (int i= 0; i < positions.length; i+= 1) {
			IntVar variable= variables[i];
			Position position= positions[i];
			Section section= position.getSection();
			for (int j = 0; j < assistants.length; j += 1) {
				Assistant anAssistant = assistants[j];
				if (anAssistant.unavailableFor(section.getTime())) {
					jacopStore.impose(new XneqC(variable,j));
				}
			}
		}
	}

}
