package tlass.project.constraintTypes;

import tlass.project.model.Assistant;
import tlass.project.model.Constraint;
import tlass.project.model.Position;
import tlass.project.model.Section;
import JaCoP.constraints.XneqY;
import JaCoP.core.IntVar;
import JaCoP.core.Store;

public class PreventSectionConflictsConstraint extends Constraint{

	/**
	 * Constructor for this constraint.
	 */
	public PreventSectionConflictsConstraint() {
		name= "Prevent Section Conflicts";
		description= "";
	}
	
	/**
	 * Avoid Assignment Time Conflicts: If an assistant
	 * is assigned to two different positions, those 
	 * positions times must not intersect.
	 */
	@Override
	public void imposeConstraint(Store jacopStore, IntVar[] variables,
			Position[] positions, Assistant[] assistants) {
		for (int i= 0; i < positions.length; i+= 1) {
			IntVar firstVariable= variables[i];
			Position firstPosition= positions[i];
			Section firstSection= firstPosition.getSection();
			for (int j= 0; j < positions.length; j+= 1) {
				IntVar secondVariable= variables[j];
				Position secondPosition= positions[j];
				Section secondSection= secondPosition.getSection();
				if (firstSection != secondSection) {
					if (firstSection.intersect(secondSection)) {
						jacopStore.impose(new XneqY(firstVariable,secondVariable));						
					}
				} else {
					if (firstPosition != secondPosition) {
						jacopStore.impose(new XneqY(firstVariable, secondVariable));
					}
				}
			}
		}
	}

}
