package tlass.project.constraintTypes;

import java.util.HashSet;

import tlass.project.model.Assistant;
import tlass.project.model.Constraint;
import tlass.project.model.Position;
import tlass.project.model.Section;
import tlass.project.model.Skill;

import JaCoP.constraints.XneqC;
import JaCoP.core.IntVar;
import JaCoP.core.Store;

public class RequireTagsConstraint extends Constraint{

	/**
	 * Constructor for this constraint.
	 */
	public RequireTagsConstraint() {
		name= "Require Tags";
		description= "";
	}
	
	/**
	 * Require Skills: To be assigned to a position, 
	 * an assistant must have at least all the skills 
	 * that the positions section requires.
	 */
	@Override
	public void imposeConstraint(Store jacopStore, IntVar[] variables,
			Position[] positions, Assistant[] assistants) {
		for (int i= 0; i < positions.length; i+= 1) {
			IntVar aVariable= variables[i];
			Position aPosition= positions[i];
			Section section= aPosition.getSection();
			for (int j= 0; j < assistants.length; j+= 1) {
				Assistant anAssistant= assistants[j];
				HashSet<Skill> skillsNeeded= section.getSkillsNeeded();
				HashSet<Skill> skillsHave= anAssistant.getSkills();
				if (!skillsHave.containsAll(skillsNeeded)) {
					jacopStore.impose(new XneqC(aVariable, j));
				}
			}
		}
	}

}
