package tlass.project.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import tlass.project.Controller;
import tlass.project.ModelHandler;
import tlass.project.Pair;
import tlass.project.WebformHandler;
import tlass.project.model.Assistant;
import tlass.project.model.Model;
import tlass.project.model.TimeBlock;


public class AuditDialog extends JDialog {
	private static final long serialVersionUID = 6084763258312489518L;

	private Controller controller;
	private WebformHandler webformHandler;
	private ModelHandler modelHandler;
	private JFrame owner;
	private Model model;
	private AuditDialog self; // Used to reference this inside of an anonymous inner
						// class
	private ArrayList<Pair<String, TimeBlock>> times;
	private ArrayList<AuditCheckBox> audits;

	/**
	 * Constructor for Audit Dialog
	 * 
	 * @param JFrame of the window
	 * @param Controller where the model, model handler and webform handlers are
	 * @param ArrayList of times (String, TimeBlock)
	 */
	public AuditDialog(JFrame o, Controller c,
			ArrayList<Pair<String, TimeBlock>> times) {
		super(o, true);
		this.controller = c;
		this.owner = o;
		this.self = this;
		this.model = c.getModel();
		this.modelHandler = c.getModelHandler();
		webformHandler = c.getWebformHandler();
		this.times = times;

		rootPane.setLayout(new BorderLayout());

		JPanel toolBar = new JPanel();
		//accept all button
		JButton acceptButton = new JButton("Accept All");
		acceptButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				for (AuditCheckBox anAudit : audits) {
					anAudit.setSelected(true);
				}
			}
		});
		//reject all button
		JButton rejectButton = new JButton("Reject All");
		rejectButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				for (AuditCheckBox anAudit : audits) {
					anAudit.setSelected(false);
				}
			}
		});
		toolBar.add(acceptButton);
		toolBar.add(rejectButton);
		rootPane.add(toolBar, BorderLayout.PAGE_START);

		JPanel timeTable = new JPanel();
		BoxLayout layout = new BoxLayout(timeTable, BoxLayout.Y_AXIS);
		timeTable.setLayout(layout);
		String previousUsername = "";
		audits = new ArrayList<AuditCheckBox>();
		//for every time pair given
		for (Pair<String, TimeBlock> aPair : times) {
			//breakup the pair
			String username = aPair.getFirst();
			TimeBlock time = aPair.getSecond();
			//get the assistant by username
			Assistant assistant = model.getAssistantByUsername(username);
			//if the username is not the last add a row for it
			if (!username.equals(previousUsername)) {
				JPanel row = new JPanel();
				JLabel userLabel = new JLabel(assistant.getName());
				userLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
				row.add(userLabel);
				timeTable.add(row);
			}
			previousUsername = username;
			JPanel row = new JPanel();
			JTextArea commentLabel = makeLabelStyle(new JTextArea(3,30));
			commentLabel.setLineWrap(true);
			commentLabel.setText(time.getTimeString()+"\n\""+time.getDescription()+'"');
			row.add(commentLabel);
			AuditCheckBox acceptBox = new AuditCheckBox(assistant, time);
			row.add(acceptBox.getCheckBox());
			audits.add(acceptBox);
			timeTable.add(row);
		}

		JScrollPane tableScroller = new JScrollPane(timeTable);
		rootPane.add(tableScroller, BorderLayout.CENTER);

		JPanel confirmBar = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				self.dispose();
			}
		});
		JButton okayButton = new JButton("Finished Auditing");
		okayButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				//for every audit that is selected add the inavailable time to the model handler
				for (AuditCheckBox anAudit : audits) {
					if (anAudit.isSelected()) {
						modelHandler.addUnavailableTime(anAudit.getAssistant(),
								anAudit.getTime());
					}
				}
				try {
					webformHandler.removeUnavailableTimes(model.getSemester());
				} catch (Exception e1) {
					// Better just hope it doesn't happen;
				}
				self.dispose();
			}
		});
		confirmBar.add(cancelButton);
		confirmBar.add(okayButton);
		rootPane.add(confirmBar, BorderLayout.PAGE_END);

		// Display all the users and times
		setSize(500, 600);
		setLocationRelativeTo(null);
		setVisible(true);
	}

	/**
	 * Static function that will style a JTextArea to look like a JLabel 
	 * Taken from http://www.coderanch.com/t/338648/GUI/java/Multiple-lines-JLabel
	 * @param textArea
	 * @return JTextArea
	 */
	public static JTextArea makeLabelStyle(JTextArea textArea) {
		if (textArea == null)
			return null;
		textArea.setEditable(false);
		textArea.setCursor(null);
		textArea.setOpaque(false);
		textArea.setFocusable(false);
		return textArea;
	}

	private class AuditCheckBox {
		JCheckBox checkBox;
		private Assistant assistant;
		private TimeBlock time;

		/**
		 * create checkbox and set and assistant and time for it
		 * @param Assistant
		 * @param TimeBlock
		 */
		public AuditCheckBox(Assistant a, TimeBlock t) {
			checkBox = new JCheckBox("Accept", false);
			setAssistant(a);
			setTime(t);
		}

		/**
		 * getter for checkbox
		 * @return checkbox
		 */
		public Component getCheckBox() {
			return checkBox;
		}
		
		/**
		 * set selected true/false
		 * @param Boolean state
		 */
		public void setSelected(boolean state) {
			checkBox.setSelected(state);
		}

		/**
		 * getter for if checkbox if selected or not 
		 * @return boolean
		 */
		public boolean isSelected() {
			return checkBox.isSelected();
		}

		/**
		 * setter for asistant
		 * @param Assistant
		 */
		public void setAssistant(Assistant assistant) {
			this.assistant = assistant;
		}

		/**
		 * getter for assistant
		 * @return Assistant
		 */
		public Assistant getAssistant() {
			return assistant;
		}

		/**
		 * setter for time
		 * @param TimeBlock
		 */
		public void setTime(TimeBlock time) {
			this.time = time;
		}

		/**
		 * getter for time
		 * @return TimeBlock
		 */
		public TimeBlock getTime() {
			return time;
		}
	}
}
