/* LoginWindow.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import tlass.project.*;
import tlass.project.exceptions.DbConnectionException;


public class LoginWindow extends JPanel {
	private static final long serialVersionUID = 1L;

	private JTextField userEntry;
	private JPasswordField passEntry;
	private JLabel message;
	private JButton loginButton;

	private View view;
	private LoginHandler loginHandler;
	private User user;

	private JFrame owner;

	/**
	 * Build graphical elements and connect them to the Login system The
	 * hostController will be notified when we're logged in
	 */
	public LoginWindow(JFrame o, Controller hostController) {
		this.setOpaque(true);
		this.loginHandler = hostController.getLoginHandler();
		this.user = hostController.getUser();
		this.view = hostController.getView();
		this.owner = o;

		// setup content holders for in the frame
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		// Build Username component
		JPanel userGroup = new JPanel();
		userGroup.setLayout(new BorderLayout());
		userGroup.add("West", new JLabel(" Username:  "));
		userEntry = new JTextField();
		userGroup.add("Center", userEntry);
		add(userGroup);

		// Build Password component
		JPanel passGroup = new JPanel();
		passGroup.setLayout(new BorderLayout());
		passGroup.add("West", new JLabel(" Password:  "));
		passEntry = new JPasswordField();
		passGroup.add("Center", passEntry);
		add(passGroup);

		// Build Login button
		JPanel msgGroup = new JPanel();
		loginButton = new JButton("Login");
		msgGroup.add(loginButton);
		message = new JLabel("");
		msgGroup.add(message);
		add(msgGroup);

		// Action for when user attempts to login
		ActionListener loginAction = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				//set the username and password
				user.setUsername(userEntry.getText());
				user.setPassword(passEntry.getPassword());
				try {
					//if the user logs in successfully
					if (loginHandler.checkUser(user)) {
						//complete the login
						//loginHandler.loginUser(user);
						//close the login window
						view.closeLoginWindow();
						//open the semester dialog box
						view.openSemesterDialog();
					} else {
						//tell the user that they put the wrong login in
						JOptionPane
						.showMessageDialog(
								owner,
								"Your login credentials were rejected.",
								"Invalid Login",
								JOptionPane.ERROR_MESSAGE);
					}
				} catch (DbConnectionException e) {
					//tell the user the database connection could not be made
					String[] messages = {
							"The system was unable to connect to the server. Please try again later.",
							e.getMessage() };
					JOptionPane.showMessageDialog(owner, messages,
							"Connection error", JOptionPane.ERROR_MESSAGE);
				}
			}
		};
		//add the action listener to the button and text boxes
		loginButton.addActionListener(loginAction);
		passEntry.addActionListener(loginAction);
		userEntry.addActionListener(loginAction);
	}

	/**
	 * set the login error message
	 * @param s
	 */
	public void setLoginErrorMessage(String s) {
		message.setText(s);
	}

	/**
	 * getter for the login button
	 * @return
	 */
	public JButton getLoginButton() {
		return loginButton;
	}

	/**
	 * getter for user text field
	 * @return
	 */
	public JTextField getUser() {
		return userEntry;
	}

	/**
	 * getter for pass password field
	 * @return
	 */
	public JPasswordField getPass() {
		return passEntry;
	}
}
