/* MainMenu.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import tlass.project.Controller;
import tlass.project.LoginHandler;
import tlass.project.SemesterHandler;
import tlass.project.User;
import tlass.project.model.Model;


public class MainMenu extends JMenuBar {
	
	private static final long serialVersionUID = 1L;
	private JMenu fileMenu, viewMenu, helpMenu;
	public JMenuItem logout, exit, toolbarCB, openState, saveState, export, print, about, gettingStarted;
	public JMenuItem close, closeAll;
	private JFrame genericWindow;
	private HelpWindow helpWindow;
	
	private Controller controller;
	private LoginHandler loginHandler;
	private View view;
	private JFrame frame;
	
	/**
	 * construct the main menu
	 * @param Controller that has the login handler and view
	 */
	public MainMenu(Controller hostController) {
		this.setOpaque(true);
		this.controller= hostController;
		this.loginHandler= hostController.getLoginHandler();
		this.view= hostController.getView();
		this.frame= view.getFrame();
		
		//Build the File menu.
		fileMenu = new JMenu("File");
		// adds a keyboard shortcut
		fileMenu.setMnemonic(KeyEvent.VK_F);
		// Build the load menu item
		openState = new JMenuItem("Change Semester");
		// adds a keyboard shortcut and shows it next to the item on the gui
		openState.setMnemonic(KeyEvent.VK_O);
		openState.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.ALT_MASK));
		// Build the save menu item
		saveState = new JMenuItem("Save");
		// adds a keyboard shortcut and shows it next to the item on the gui
		saveState.setMnemonic(KeyEvent.VK_S);
		saveState.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
		// Build the export menu item
		export = new JMenuItem("Export to File...");
		// Build the print menu item
		print = new JMenuItem("Print");
		// adds a keyboard shortcut and shows it next to the item on the gui
		print.setMnemonic(KeyEvent.VK_P);
		print.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.ALT_MASK));
		
		// Create the close menu item
		close = new JMenuItem("Close");
		// adds a keyboard shortcut and shows it next to the item on the gui
		close.setMnemonic(KeyEvent.VK_W);
		close.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.ALT_MASK));
		closeAll = new JMenuItem("Close All");
				
		// Create the logout menu item
		logout = new JMenuItem("Logout",KeyEvent.VK_L);
		// adds a keyboard shortcut and shows it next to the item on the gui
		logout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.ALT_MASK));
		logout.getAccessibleContext().setAccessibleDescription("Logout of the program");
		// Create the exit menu item
		exit = new JMenuItem("Exit");
		// add a keyboard shortcut
		exit.setMnemonic(KeyEvent.VK_E);
		
		// add all of the menu items to file menu
		fileMenu.add(openState);
		fileMenu.add(saveState);
		fileMenu.addSeparator();
		fileMenu.add(export);
		fileMenu.add(print);
		fileMenu.addSeparator();
		fileMenu.add(close);
		fileMenu.add(closeAll);
		fileMenu.addSeparator();
		fileMenu.add(logout);
		fileMenu.addSeparator();
		fileMenu.add(exit);
		add(fileMenu);
	
		// Build the view menu in the menu bar
		viewMenu = new JMenu("View");
		viewMenu.setMnemonic(KeyEvent.VK_V);
		// Create the checkbox menuitem that will toggle if toolbars can move
		toolbarCB = new JCheckBoxMenuItem("Lock Toolbar", true);
		viewMenu.add(toolbarCB);
		add(viewMenu);	
		
		// Build the help menu in the menu bar
		helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
		// Create the about window that will display information about the project
		about = new JMenuItem("About");
		helpMenu.add(about);
		add(helpMenu);
		
		
		// ACTION LISTENERS
		// ActionListener for when user attempts to login
		logout.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				User user= controller.getUser();
				//loginHandler.logoutUser(user);
				view.closeMainWindow();
				view.openLoginWindow();
			}
			
		});
		
		// ActionListener for when user attempts to exit
		exit.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {	
				System.exit(0);
			}
			
		});
		
		// ActionListener for when the user tries to load a saved state
		openState.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				view.closeMainWindow();
				view.openSemesterDialog();
			}
		});
		
		// ActionListener for when the user tries to save the current state
		// TODO
		saveState.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				Model model = controller.getModel();
				SemesterHandler semesterHandler= controller.getSemesterHandler();
				try {
					semesterHandler.saveSemester(model.getSemester(), model);
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(
							null,
							"Could not save the changes. Try again later.",
							"Network Error",
							JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		
		// ActionListener for when the user wishes to export the schedule received
		// TODO
		export.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				System.out.println("Export the data");
				
			}
		});
		
		// ActionListener for when the user wishes to print the schedule
		// TODO
		print.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				System.out.println("Print the data");
				
			}
		});
		
		// Add the functionality to toggle the view of the toolbar at the top
		// TODO
		toolbarCB.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				System.out.println("Toggle toolbars");
				
			}
		});
		
		// ActionListener for the about button. This opens a dialog that displays information
		// pertaining to the program and authors etc.
		// TODO
		about.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				createHelpWindow();
			}
		});

	}
	/**
	 * create the help window to display to the user
	 */
	private void createHelpWindow(){
		genericWindow = new JFrame("About the Teaching and Lab Assistant Scheduling System");
		genericWindow.setLayout(new BorderLayout());
		genericWindow.setMinimumSize(new Dimension(300, 100));
		
		//Create the help window and add this to our JFrame
		helpWindow = new HelpWindow();
		
		genericWindow.add(helpWindow, BorderLayout.CENTER);
		genericWindow.pack();
		// center the frame on screen
		genericWindow.setLocationRelativeTo(null);
		genericWindow.setVisible(true);		
	}
}
	
	

