/* MainWindow.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.gui;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import javax.swing.JTree;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import tlass.project.*;
import tlass.project.exceptions.DbConnectionException;
import tlass.project.gui.panels.AssistantPanel;
import tlass.project.gui.panels.ConstraintPanel;
import tlass.project.gui.panels.InstructorPanel;
import tlass.project.gui.panels.ResourcePanel;
import tlass.project.gui.panels.SchedulePanel;
import tlass.project.gui.panels.SectionPanel;
import tlass.project.model.Assistant;
import tlass.project.model.Constraint;
import tlass.project.model.Instructor;
import tlass.project.model.Model;
import tlass.project.model.Position;
import tlass.project.model.Schedule;
import tlass.project.model.Section;
import tlass.project.model.Skill;
import tlass.project.model.TimeBlock;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;


public class MainWindow extends JPanel {
	private static final long serialVersionUID = 1L;
	public JToolBar toolBar;
	private JTree tree;
	private DefaultTreeModel treeModel;
	public JTabbedPane tabPane;
	
	private Controller controller;
	private WebformHandler webformHandler;
	
	public ArrayList<Object> openTabs;
	private JPopupMenu menu = new JPopupMenu("Operations");
	JFrame frame = new JFrame();
	DefaultTreeCellRenderer renderer;
	ImageIcon[] leafIcons;
	DefaultComboBoxModel assistantsModel;
	JCheckBox activeSemester;
	
	HashMap<Object, DefaultMutableTreeNode> treeResourceMap;
	
	public DefaultComboBoxModel getAssistantsModel(){
		return assistantsModel;		
	}
	
	public void setAssistantsModel(DefaultComboBoxModel a){
		this.assistantsModel = a;
	}

	private static final int ASSISTANT_RESOURCE= 0;
	private static final int INSTRUCTOR_RESOURCE= 1;
	private static final int SECTION_RESOURCE= 2;
	private static final int CONSTRAINT_RESOURCE= 3;
	private static final int SCHEDULE_RESOURCE= 4;
	private static final int RESOURCE_TYPES= 5;
	private DefaultMutableTreeNode topCategory;
	private DefaultMutableTreeNode[] resourceCategories;
	private ModelHandler modelHandler;
	private Scheduler scheduler;
	private Model model;
	private JFrame genericWindow;

	/**
	 * Constructor
	 * @param c
	 */
	public MainWindow(Controller c) {
		super(new BorderLayout());
		this.setOpaque(true);
		this.controller= c;
		this.modelHandler= c.getModelHandler();
		this.webformHandler = c.getWebformHandler();
		this.model= c.getModel();

		//Create the toolbar
		this.toolBar= new JToolBar();
		toolBar.setFloatable(false);

		ModelListener forceRepaint= new ModelListener() {
			@Override
			public void onEvent(ModelEvent event) {
				if (event.getType() == ModelEvent.RENAMED) {
					DefaultMutableTreeNode nodeChanged= treeResourceMap.get(event.getItem());
					treeModel.nodeChanged(nodeChanged);
				}
			}
		};
		modelHandler.addAssistantsListener(forceRepaint);
		modelHandler.addSectionsListener(forceRepaint);
		modelHandler.addInstructorsListener(forceRepaint);
		modelHandler.addSchedulesListener(forceRepaint);

		// Creates the run button and attatches an ActionListner
		JButton pickScheduleButton= new JButton("Run");
		pickScheduleButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int action= checkAudit();
				if (action == 1) {
					try {
						ArrayList<Pair<String,TimeBlock>> times= webformHandler.getAllUnavailableTimes(model.getSemester());
						AuditDialog ad = new AuditDialog(frame, controller, times);
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
				if (action != 0) {
					Schedule s= modelHandler.pickNewSchedule();
					if (s == null) {
						int result= JOptionPane.showConfirmDialog(frame,
								"Couldn't find a valid schedule. Do you want the Scheduler to look "+
								"for problem positions?",
								"Diagnose Schedule",
								JOptionPane.YES_NO_OPTION);
						if (result == 0) {
							ArrayList<Position> problemPositions= modelHandler.findProblemPositions();
							String[] messages= new String[2+problemPositions.size()];
							messages[0]= "The following positions are completely impossible to fill as is.";
							for (int i= 0; i < problemPositions.size(); i+= 1) {
								messages[1+i]= problemPositions.get(i).getId();
							}
							messages[1+problemPositions.size()]= "Try to loosen any restrictions on them.";
							if (!problemPositions.isEmpty()) {
								JOptionPane.showMessageDialog(frame, messages);
							} else {
								JOptionPane.showMessageDialog(frame, "The scheduler couldn't find any obvious problems with the schedule. Try removing constraints and running the scheduler again.");
							}
						}
					} else {
						DefaultMutableTreeNode newResource = new DefaultMutableTreeNode(s);
						DefaultMutableTreeNode category= resourceCategories[SCHEDULE_RESOURCE];
						treeResourceMap.put(s, newResource);
						treeModel.insertNodeInto(newResource, category, category.getChildCount());
					}
				}
			}

			/**
			 * Creates a dialog that prompts the user to
			 * either audit the assignments or run the scheduler
			 * @return
			 */
			private int checkAudit() {
				Object[] options = {"Cancel",
				                    "Audit",
				                    "Run Scheduler"};
				return JOptionPane.showOptionDialog(frame,
				    "Do you want to check for new times that Assistants are unavailable " +
				    "before you run the scheduler?",
				    "Audit before Scheduling",
				    JOptionPane.YES_NO_CANCEL_OPTION,
				    JOptionPane.QUESTION_MESSAGE,
				    null, options, options[2]);
			}
		});
		toolBar.add(pickScheduleButton);

		// Creates the button that will add a new assistant and attaches an actionListner
		JButton addAssistantButton= new JButton("Add Assistant");
		addAssistantButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Assistant a= modelHandler.addNewAssistant();
				String name= createNewWindow();
				if (name != null) {
					a.setName(name);
					DefaultMutableTreeNode newResource = new DefaultMutableTreeNode(a);
					DefaultMutableTreeNode category= resourceCategories[ASSISTANT_RESOURCE];
					treeResourceMap.put(a, newResource);
					treeModel.insertNodeInto(newResource, category, category.getChildCount());
				}
			}
		});
		toolBar.add(addAssistantButton);

		// Creates the button that will add a section and attaches an actionListener
		JButton addSectionButton= new JButton("Add Section");
		addSectionButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Section s= modelHandler.addNewSection();
				String name= createNewWindow();
				if (name != null) {
					s.setId(name);
					DefaultMutableTreeNode newResource = new DefaultMutableTreeNode(s);
					treeResourceMap.put(s, newResource);
					DefaultMutableTreeNode category= resourceCategories[SECTION_RESOURCE];
					treeModel.insertNodeInto(newResource, category, category.getChildCount());
				}
			}
		});
		toolBar.add(addSectionButton);
		
		// Creates a button that will add a new Instructor and attaches an actionListener.
		JButton addInstructorButton= new JButton("Add Instructor");
		addInstructorButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Instructor i= modelHandler.addNewInstructor();
				String name= createNewWindow();
				if (name != null) {
					i.setName(name);
					DefaultMutableTreeNode newResource = new DefaultMutableTreeNode(i);
					treeResourceMap.put(i, newResource);
					DefaultMutableTreeNode category= resourceCategories[INSTRUCTOR_RESOURCE];
					treeModel.insertNodeInto(newResource, category, category.getChildCount());
				}
			}
		});
		toolBar.add(addInstructorButton);
		
		// Creates a button that will allow the user to audit available times
		// and adds an actionListener
		JButton auditTimesButton = new JButton("Audit Unavailable Times");
		auditTimesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					ArrayList<Pair<String,TimeBlock>> times= webformHandler.getAllUnavailableTimes(model.getSemester());
					AuditDialog ad = new AuditDialog(frame, controller, times);
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		});
		toolBar.add(auditTimesButton);

		// Creates a button that close the tab and adds an actionListener
		JButton closeTabButton = new JButton("Close Tab");
		closeTabButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int index= tabPane.getSelectedIndex();
				closeTab(index);
			}
		});
		toolBar.add(closeTabButton);
		
		// Creates a button that will remove a selected resource and
		// adds an actionListener.
		JMenuItem removeModel = new JMenuItem("Remove");
		removeModel.setToolTipText("Removes selected resource from the model");
		removeModel.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				TreePath currentSelection = tree.getSelectionPath();
				if (currentSelection != null) {
					// Identify the resource
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) currentSelection.getLastPathComponent();
					DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) tree.getModel().getRoot();
					if (!rootNode.isNodeChild(node)) {
						// Remove the resource from the graphical tree
						DefaultTreeModel model = ((DefaultTreeModel) tree.getModel());
						treeResourceMap.remove(node);
						model.removeNodeFromParent(node);
						closeTab(node.getUserObject());
						// Remove the resource from the model
						modelHandler.removeResource(node.getUserObject());
					}
				}
			}
		});
		menu.add(removeModel);
		menu.addSeparator();

		JMenuItem closeAllTabs = new JMenuItem("Close All");
		closeAllTabs.setToolTipText("Close all open tabs");
		menu.add(closeAllTabs);

		JMenuItem rename = new JMenuItem("Rename");
		rename.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent argo0){
				String name = (String) JOptionPane.showInputDialog(frame,
						"Enter name", "Rename ",
						JOptionPane.PLAIN_MESSAGE, null, null, null);
				if (name == null) return;
				TreePath currentSelection = tree.getSelectionPath();
				if (currentSelection != null) {
					// Identify the resource
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) currentSelection.getLastPathComponent();
					DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) tree.getModel().getRoot();
					if (!rootNode.isNodeChild(node)) {
						// Rename the resource from the graphical tree
						DefaultTreeModel model = ((DefaultTreeModel) tree.getModel());
						modelHandler.renameResource(node.getUserObject(), name);
						treeModel.nodeChanged(node);
					}
				}
			}
		});
		rename.setToolTipText("Edit the name of the tab");
		menu.add(rename);
		
		activeSemester= new JCheckBox();
		activeSemester.setSelected(model.isActive());
		activeSemester.setText("Active Semester");
		activeSemester.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				try {
					String action;
					if (!activeSemester.isSelected()) {
						action= "Deactivate";
					} else {
						action= "Activate";
					}
					int result= JOptionPane.showConfirmDialog(frame,
							"Are you sure you want to "+action+" this schedule?",
							action+" schedule",
							JOptionPane.YES_NO_OPTION);
					if (result == 0) {
						controller.getSemesterHandler().setActiveSemester(model.getSemester(), activeSemester.isSelected());
						model.setActive(activeSemester.isSelected());
					} else {
						activeSemester.setSelected(model.isActive());
					}
				} catch (Exception e) {
					e.printStackTrace();
					activeSemester.setSelected(model.isActive());
					Auxiliary.showConnectionError();
				}
			}
			
		});
		
		toolBar.add(activeSemester);


		this.add(toolBar, BorderLayout.PAGE_START);
		this.renderer = new CustomLeafIconRenderer();
		
		treeResourceMap= new HashMap<Object, DefaultMutableTreeNode>();

		//Create the nodes.
		topCategory = new DefaultMutableTreeNode("Assistant Scheduling System Editor");
		createCategories();
		treeModel = new DefaultTreeModel(topCategory);

		//Create a tree that allows one selection at a time.
		tree = new JTree(treeModel);
		tree.setRootVisible(false);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setCellRenderer(renderer);
		JScrollPane treePane = new JScrollPane(tree);

		// Create the tabs
		tabPane = new JTabbedPane();
		openTabs= new ArrayList<Object>();

		//Add the scroll panes to a split pane.
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setLeftComponent(treePane);
		splitPane.setRightComponent(tabPane);

		// Adjust split pane sizes
		Dimension minimumSize = new Dimension(100, 50);
		tabPane.setMinimumSize(minimumSize);
		treePane.setMinimumSize(minimumSize);
		splitPane.setDividerLocation(200);
		splitPane.setPreferredSize(new Dimension(500, 300));		
		

		//Add the split pane to this panel.
		add(splitPane, BorderLayout.CENTER);

		MouseAdapter treeMenuClicked = new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				if(e.getClickCount() == 2 && e.getButton() == MouseEvent.BUTTON1) {
					int row = tree.getClosestRowForLocation(e.getX(), e.getY());
					tree.setSelectionPath(tree.getPathForRow(row));
					DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
					// When a non-resource is selected, break out
					if (selectedNode == null || !selectedNode.isLeaf()) {
						return;
					}

					// Constraints are singleton tabs
					Object resource = selectedNode.getUserObject();
					int openedConstraintIndex= -1;
					if (resource instanceof Constraint) {
						int i= 0;
						for (Object aResource : openTabs) {
							if (aResource instanceof Constraint) {
								openedConstraintIndex= i;
								break;
							}
							i+= 1;
						}
					}
					
					if (openedConstraintIndex != -1) {
						tabPane.setSelectedIndex(openedConstraintIndex);
					} else if (openTabs.contains(resource)) {


					} else {
						openTabs.add(resource);
						ResourcePanel resourcePanel= buildPanel(resource, selectedNode);
						if (resourcePanel != null) {
							String name= resource.toString();
							if (resource instanceof Constraint) {
								name= "Constraints";
							}
							tabPane.add(name, resourcePanel);
							int tabCount = tabPane.getTabCount();
							tabPane.setSelectedIndex(tabCount-1);
						}
					}
				} else if (e.getButton() == MouseEvent.BUTTON2) {
					maybeShowPopup(e);
				}
			}

			public void mouseReleased(MouseEvent e) {
				maybeShowPopup(e);
			}

			private void maybeShowPopup(MouseEvent e) {
				if (e.isPopupTrigger()) {
					int row = tree.getClosestRowForLocation(e.getX(), e.getY());
					tree.setSelectionPath(tree.getPathForRow(row));
					menu.show(tree, e.getX(), e.getY());
				}
			}
		};
		// Add a mouse listener to the tree
		tree.addMouseListener(treeMenuClicked);
		
		tree.addTreeSelectionListener(new TreeSelectionListener() {
			@Override
			public void valueChanged(TreeSelectionEvent e) {
				
			}
		});
	}
	
	/**
	 * Given a Resource, closes an open tab associated with it. It is
	 * fine if there is no open tab associated with it.
	 * @param userObject
	 */
	public void closeTab(Object userObject) {
		// Remove the resource from the tabs list
		int index= openTabs.indexOf(userObject);
		openTabs.remove(userObject);
		// Remove the resource from the actual graphical tabs
		if (index != -1) {
			tabPane.remove(index);
		}
	}
	
	/**
	 * Given a tab index, closes that specific tab. 
	 * @param index
	 */
	public void closeTab(int index) {
		if (tabPane.getTabCount()>0 && index != -1){
			openTabs.remove(index);
			tabPane.remove(index);
		}
	}

	/**
	 * Builds an appropriate panel for a resource
	 * Not great OO, but good for MVC. Resources must have no idea about ResourcePanels
	 * @param resource
	 * @param selectedNode 
	 * @return
	 */
	private ResourcePanel buildPanel(Object resource, DefaultMutableTreeNode selectedNode) {
		if (resource instanceof Assistant) {
			return new AssistantPanel(controller, (Assistant)resource, tree, selectedNode, this);
		} else if (resource instanceof Section) {
			return new SectionPanel(controller, (Section)resource, tree, selectedNode, this);
		} else if (resource instanceof Instructor) {
			return new InstructorPanel(controller, (Instructor)resource, tree, selectedNode, this);
		} else if (resource instanceof Constraint) {
			return new ConstraintPanel(controller, (Constraint)resource);
		} else if (resource instanceof Schedule) {
			return new SchedulePanel(this, controller, (Schedule)resource);
		} else {
			return null;
		}
	}

	@SuppressWarnings("rawtypes")
	/**
	 * Builds the categories by consuming an a name and ArrayList of resources
	 * Each of the resources becomes a child of the category that is named name.
	 */
	private DefaultMutableTreeNode buildCategory(String name, ArrayList resources) {
		DefaultMutableTreeNode category;
		//all other resources are in folders
		if(name!="Constraints") {
			category = new DefaultMutableTreeNode(name);
			for (Object aResource : resources) {
				DefaultMutableTreeNode aResourceNode= new DefaultMutableTreeNode(aResource, false);
				treeResourceMap.put(aResource, aResourceNode);
				category.add(aResourceNode);
			}
		} else {//there is only one instance of a constraint panel
			Constraint c = (Constraint) resources.get(0);
			c.setName("Constraints");
			category = new DefaultMutableTreeNode(c, false);
			treeResourceMap.put(resources.get(0), category);
		}

		return category;
	}

	/**
	 * Creates the categories (folders that will have children)
	 */
	private void createCategories() {
		resourceCategories= new DefaultMutableTreeNode[RESOURCE_TYPES];
		resourceCategories[SECTION_RESOURCE]= buildCategory("Sections", model.getSections());
		resourceCategories[ASSISTANT_RESOURCE]= buildCategory("Assistants", model.getAssistants());
		resourceCategories[INSTRUCTOR_RESOURCE]= buildCategory("Instructors", model.getInstructors());
		resourceCategories[CONSTRAINT_RESOURCE]= buildCategory("Constraints", model.getConstraints());
		resourceCategories[SCHEDULE_RESOURCE]= buildCategory("Schedules", model.getSchedules());

		for (DefaultMutableTreeNode aCategory : resourceCategories) {
			topCategory.add(aCategory);
		}
	}

	/**
	 * Gets the tree.
	 * @return
	 */
	public JTree getTree() {
		return tree;
	}

	/**
	 * Gets the controller.
	 * @return
	 */
	public Controller getController(){
		return controller;
	}
	
	String createNewWindow(){
		return (String) JOptionPane.showInputDialog(frame,
				"Enter name", "New Resource",
				JOptionPane.PLAIN_MESSAGE, null, null, null);
	}

	/**
	 * Responsible for creating the icons that are displayed on the leaf nodes.
	 * @author E
	 *
	 */
	private class CustomLeafIconRenderer extends DefaultTreeCellRenderer {
		private static final long serialVersionUID = -8941283336711978300L;
		private ImageIcon[] leafIcons;

		public CustomLeafIconRenderer() {
			leafIcons= new ImageIcon[RESOURCE_TYPES];
			leafIcons[ASSISTANT_RESOURCE]= createImageIcon("/resources/assistants.png");
			leafIcons[CONSTRAINT_RESOURCE]= createImageIcon("/resources/constraints.png");
			leafIcons[SCHEDULE_RESOURCE]= createImageIcon("/resources/schedules.png");
			leafIcons[SECTION_RESOURCE]= createImageIcon("/resources/sections.png");
			leafIcons[INSTRUCTOR_RESOURCE]= createImageIcon("/resources/instructors.png");
		}
		
		public Component getTreeCellRendererComponent(JTree tree,Object value,boolean sel,
				boolean expanded,boolean leaf,int row,boolean hasFocus) {
			super.getTreeCellRendererComponent(tree, value, sel,expanded, leaf, row,hasFocus);
			if (leaf) {
				setIcon(getLeafIcon(value));
			}
			return this;
		}

		/**
		 * Loads an image from the given path. Returns null on failure
		 * @param string
		 * @return
		 */
		private ImageIcon createImageIcon(String string) {
			java.net.URL imgURL = getClass().getResource(string);
			if (imgURL != null) {
				return new ImageIcon(imgURL);
			} else {
				System.err.println("Couldn't find file: "+string);
				return null;
			}
		}

		/**
		 * Identifies the appropriate leaf image for the given resource
		 * @param value
		 * @return
		 */
		protected ImageIcon getLeafIcon(Object value) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			Object nodeInfo = node.getUserObject();
			// The following is bad OO, but a necessary evil to avoid other headaches
			// At least it follows fly-weight design pattern.
			if (nodeInfo instanceof Assistant) {
				return leafIcons[ASSISTANT_RESOURCE];
			} else if (nodeInfo instanceof Section) {
				return leafIcons[SECTION_RESOURCE];
			} else if (nodeInfo instanceof Constraint) {
				return leafIcons[CONSTRAINT_RESOURCE];
			} else if (nodeInfo instanceof Schedule) {
				return leafIcons[SCHEDULE_RESOURCE];
			} else if (nodeInfo instanceof Instructor) {
				return leafIcons[INSTRUCTOR_RESOURCE];
			} else {
				return null;
			}
		}
	}

	/**
	 * Gets the tree model
	 * @return
	 */
	public DefaultTreeModel getTreeModel() {
		return treeModel;
	}

	/**
	 * Sets the tree model to the specified one.
	 * @param treeModel
	 */
	public void setTreeModel(DefaultTreeModel treeModel) {
		this.treeModel = treeModel;
	}

	/**
	 * Sets the tree to the specified tree.
	 * @param tree
	 */
	public void setTree(JTree tree) {
		this.tree = tree;
	}
}