package tlass.project.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import tlass.project.Controller;
import tlass.project.ModelHandler;
import tlass.project.Pair;
import tlass.project.SemesterHandler;
import tlass.project.WebformHandler;
import tlass.project.model.Model;
import tlass.project.model.Position;
import tlass.project.model.Response;
import tlass.project.model.Schedule;


public class SemesterWindow extends JPanel {
	private static final long serialVersionUID = 5748512339031282029L;
	Controller controller;
	ModelHandler modelHandler;
	SemesterHandler semesterHandler;
	WebformHandler webformHandler;
	JFrame owner;
	ArrayList<String> semesterList;
	JList semesterListBox;
	SemesterWindow self; // Used to reference this inside of an anonymous inner
							// class
	View view;

	/**
	 * Semester Window constructor
	 * @param View instance
	 * @param JFrame of original frame
	 * @param Controller instance
	 */
	public SemesterWindow(View v, JFrame o, Controller c) {
		super(new BorderLayout());
		this.controller= c;
		modelHandler= c.getModelHandler();
		webformHandler= c.getWebformHandler();
		this.owner= o;
		this.self= this;
		this.view= v;
		
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		semesterHandler = c.getSemesterHandler();

		try {
			semesterList = semesterHandler.getSemesters();
		} catch (Exception e) {
			semesterList = new ArrayList<String>();
			JOptionPane.showMessageDialog(
					owner,
					"Could not get the semesters.",
					"Network Error",
					JOptionPane.ERROR_MESSAGE);
		}
		add(new JLabel("Double-click one of the semesters to open it"));
		
		semesterListBox = new JList(semesterList.toArray(new String[semesterList.size()]));
		semesterListBox.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		semesterListBox.setLayoutOrientation(JList.VERTICAL);

		JScrollPane scrollPane = new JScrollPane(semesterListBox);
		add(scrollPane);

		semesterListBox.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					String selection = (String) semesterListBox
							.getSelectedValue();
					Model newModel;
					try {
						newModel = semesterHandler.loadSemester(selection);
						if (newModel == null) {
							JOptionPane.showMessageDialog(
									owner,
									"This semester is empty. Select another one or add a new semester.",
									"Corrupted Semester",
									JOptionPane.ERROR_MESSAGE);
						} else {
							modelHandler.loadModel(newModel);
							modelHandler.setSemester(selection);
							handleNewResponses(selection);
							view.closeSemesterDialog();
							view.openMainWindow();
						}
					} catch (ClassNotFoundException e1) {
						JOptionPane.showMessageDialog(
								owner,
								"This semester is corrupted. Select another one or add a new semester.",
								"Corrupted Semester",
								JOptionPane.ERROR_MESSAGE);
					} catch (Exception e1) {
						//e1.printStackTrace();
						JOptionPane.showMessageDialog(
								owner,
								"There was a connection error, couldn't load the Semester. Please try again.",
								"Connection error",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}

			/**
			 * handle any new responses
			 * @param String semester to get new responses from
			 */
			private void handleNewResponses(String semester) {
				Schedule schedule= controller.getModel().getPublishedSchedule();
				if (schedule == null) return;
				try {
					ArrayList<Pair<Position, Response>> responses= webformHandler.getNewResponses(semester, schedule);
					for (Pair<Position,Response> pair : responses) {
						Position position= pair.getFirst();
						Response response= pair.getSecond();
						modelHandler.setResponse(schedule, position, response);
					}
				} catch (Exception e) {
					//Don't worry about it
				}
			}
		});

		JButton addSemesterButton = new JButton("Add Semester");
		addSemesterButton.setAlignmentY(CENTER_ALIGNMENT);
		addSemesterButton.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				String name = (String) JOptionPane.showInputDialog(owner,
						"Enter the new Semester's name", "Semester Name",
						JOptionPane.PLAIN_MESSAGE, null, null, "New Semester");
				boolean nameIsNew = true;
				for (String aSemester : semesterList) {
					nameIsNew &= !aSemester.equals(name);
				}
				if (name != null && name.length() > 0 && nameIsNew) {
					try {
						semesterHandler.addSemester(name, controller.getModel());
						semesterList.add(name);
						semesterListBox.setListData(semesterList
								.toArray(new String[semesterList.size()]));
					} catch (Exception e){
						JOptionPane.showMessageDialog(
								owner,
								"Could not add the semester. Try again later.",
								"Connection error",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});
		add(addSemesterButton);

		JButton copySemesterButton = new JButton("Copy Semester");
		copySemesterButton.setAlignmentY(CENTER_ALIGNMENT);
		copySemesterButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				String oldSemester = (String) semesterListBox
						.getSelectedValue();
				String name = (String) JOptionPane.showInputDialog(owner,
						"Enter the new Semester's name", "Semester Name",
						JOptionPane.PLAIN_MESSAGE, null, null, "New Semester");
				boolean nameIsNew = true;
				for (String aSemester : semesterList) {
					nameIsNew &= !aSemester.equals(name);
				}
				if (name != null && name.length() > 0 && nameIsNew
						&& oldSemester != null) {
					try {
						semesterHandler.copySemester(name, oldSemester);
						semesterList.add(name);
						semesterListBox.setListData(semesterList
								.toArray(new String[semesterList.size()]));
					} catch (Exception e) {
						JOptionPane.showMessageDialog(
								owner,
								"Could not copy the semesters. Try again later.",
								"Connection error",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});
		add(copySemesterButton);

		JButton deleteSemesterButton = new JButton("Delete Semester");
		deleteSemesterButton.setAlignmentY(CENTER_ALIGNMENT);
		deleteSemesterButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				String name = (String) semesterListBox.getSelectedValue();
				if (name != null && name.length() > 0) {
					try {
						semesterHandler.deleteSemester(name);
						semesterList.remove(name);
						semesterListBox.setListData(semesterList
								.toArray(new String[semesterList.size()]));
					} catch (Exception e) {
						JOptionPane.showMessageDialog(
								owner,
								"Could not delete the semester. Try again later.",
								"Deletion blocked",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});		
		add(deleteSemesterButton);
	}
}
