package tlass.project.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import tlass.project.Controller;
import tlass.project.DataStoreConnection;
import tlass.project.LoginHandler;
import tlass.project.User;
import tlass.project.exceptions.DbConnectionException;

import au.com.bytecode.opencsv.CSVWriter;


public class ServerSettingsWindow extends JPanel {
	private JTextField hostEntry;
	private JTextField userEntry;
	private JPasswordField passEntry;
	private JTextField dbEntry;
	private JLabel message;
	private JButton saveButton;

	private View view;
	private CSVWriter writer;

	private JFrame owner;

	/**
	 * Build graphical elements and connect them to the Login system The
	 * hostController will be notified when we're logged in
	 */
	public ServerSettingsWindow(JFrame o, Controller hostController) {
		this.setOpaque(true);
		this.view = hostController.getView();
		this.owner = o;

		// setup content holders for in the frame
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		// Build Username component
		JPanel hostGroup = new JPanel();
		hostGroup.setLayout(new BorderLayout());
		hostGroup.add("West", new JLabel(" Host:  "));
		hostEntry = new JTextField(DataStoreConnection.host);
		hostGroup.add("Center", hostEntry);
		add(hostGroup);

		// Build Username component
		JPanel userGroup = new JPanel();
		userGroup.setLayout(new BorderLayout());
		userGroup.add("West", new JLabel(" Username:  "));
		userEntry = new JTextField(DataStoreConnection.user);
		userGroup.add("Center", userEntry);
		add(userGroup);

		// Build Password component
		JPanel passGroup = new JPanel();
		passGroup.setLayout(new BorderLayout());
		passGroup.add("West", new JLabel(" Password:  "));
		passEntry = new JPasswordField(DataStoreConnection.pass);
		passGroup.add("Center", passEntry);
		add(passGroup);
		
		// Build Username component
		JPanel dbGroup = new JPanel();
		dbGroup.setLayout(new BorderLayout());
		dbGroup.add("West", new JLabel(" Database:  "));
		dbEntry = new JTextField(DataStoreConnection.db);
		dbGroup.add("Center", dbEntry);
		add(dbGroup);

		// Build Login button
		JPanel msgGroup = new JPanel();
		saveButton = new JButton("Save");
		msgGroup.add(saveButton);
		message = new JLabel("");
		msgGroup.add(message);
		add(msgGroup);

		// Action for when user attempts to login
		ActionListener loginAction = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				try {
					new DataStoreConnection(hostEntry.getText(), userEntry.getText(), new String(passEntry.getPassword()), dbEntry.getText());
					//connection worked
					DataStoreConnection.host = hostEntry.getText();
					DataStoreConnection.user = userEntry.getText();
					DataStoreConnection.pass = new String(passEntry.getPassword());
					DataStoreConnection.db = dbEntry.getText();
					try {
						writer = new CSVWriter(new FileWriter("settings"));
						String[] dbStuff = (DataStoreConnection.host+"#"+DataStoreConnection.user+"#"+DataStoreConnection.pass+"#"+DataStoreConnection.db).split("#");
						writer.writeNext(dbStuff);
						writer.close();
						view.closeServerSettingsWindow();
						view.openLoginWindow();
					} catch (IOException e) {
						JOptionPane.showMessageDialog(
							owner,
							"Cannot write settings file to file system",
							"Invalid Login",
							JOptionPane.ERROR_MESSAGE);
					}
				} catch (DbConnectionException e) {
					JOptionPane.showMessageDialog(owner, e.getMessage(),
							"Database connection error", JOptionPane.ERROR_MESSAGE);
				}
			}
		};

		saveButton.addActionListener(loginAction);
		hostEntry.addActionListener(loginAction);
		userEntry.addActionListener(loginAction);
		passEntry.addActionListener(loginAction);
		dbEntry.addActionListener(loginAction);
	}

	// set the login error message
	public void setLoginErrorMessage(String s) { message.setText(s); }

	// getter for the login button
	public JButton getSaveButton() { return saveButton; }
	
	// getter for user text field
	public JTextField getHost() { return hostEntry; }

	// getter for user text field
	public JTextField getUser() { return userEntry; }

	// getter for pass password field
	public JPasswordField getPass() { return passEntry; }
	
	// getter for user text field
	public JTextField getDb() { return dbEntry; }
}
