/* View.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.gui;

import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.UIManager;

import tlass.project.*;


public class View {
	
	private final int MAXIMUM_WINDOW_WIDTH= 1000;
	private final int MAXIMUM_WINDOW_HEIGHT= 1000;
	private static JFrame frame;
	private GraphicsEnvironment env;
	
	private MainWindow mainWindow;
	private MainMenu mainMenu;
	
	private SemesterWindow semesterWindow;
	
	private ServerSettingsWindow serverSettingsWindow;
	
	private LoginWindow loginWindow;
	private LoginMenu loginMenu;
	
	private Controller controller;
	private ModelHandler modelHandler;
	
	/**
	 * Constructor for the View
	 * 
	 * @param Controller that has a model to get
	 */
	public View(Controller controller){
		this.controller= controller;
		this.modelHandler= controller.getModelHandler();
		
		//setup frame
		setFrame(new JFrame("Teaching and Lab Assistant Scheduling System"));
		JFrame.setDefaultLookAndFeelDecorated(true);
		getFrame().setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		//makes enter work with jButton
		UIManager.put("Button.defaultButtonFollowsFocus", Boolean.TRUE);		
	}
	
	/**
	 * Repaint the frame
	 */
	public static void repaint() {
		getFrame().repaint();
	}
	
	/**
	 * Hide the window
	 */
	public void hide() {
		getFrame().setVisible(false);
	}
	
	/**
	 * Show the window
	 */
	public void show() {
		getFrame().setVisible(true);
	}
	
	/**
	 * Open the server settings window
	 */
	public void openServerSettingsWindow() {
		// Link loginWindow to the host Controller
		serverSettingsWindow = new ServerSettingsWindow(getFrame(), controller);

		// Manipulate window
		getFrame().setSize(300,180);
		getFrame().setLocationRelativeTo(null);
		
		//add the window to the frame
		getFrame().add(serverSettingsWindow);
		getFrame().setVisible(true);
	}
	
	/**
	 * Open the Login Window
	 */
	public void openLoginWindow() {
		// Link loginWindow to the host Controller
		loginWindow = new LoginWindow(getFrame(), controller);

		// Manipulate window
		getFrame().setSize(300,150);
		getFrame().setLocationRelativeTo(null);

		// Give it a frame
		loginMenu = new LoginMenu(controller);
		getFrame().setJMenuBar(loginMenu);
		
		//add the window to the frame
		getFrame().add(loginWindow);
		getFrame().setVisible(true);
	}
	
	/**
	 * setup window for use when we are logged in
	 */
	public void openMainWindow() {
		mainWindow = new MainWindow(controller);

		// Add menu
		mainMenu = new MainMenu(controller);
		getFrame().setJMenuBar(mainMenu);
		
		// Adjust size and position
		maximizeWindow();
		getFrame().setLocationRelativeTo(null);
		
		//add the window to the frame
		getFrame().add(mainWindow);
		addListeners();
		getFrame().setVisible(true);
	}

	/**
	 * Resize the window if needed to the max size or size of screen whichever is larger
	 */
	private void maximizeWindow() {
		env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		getFrame().setMaximizedBounds(env.getMaximumWindowBounds());
		int currentWidth= env.getMaximumWindowBounds().width;
		int currentHeight= env.getMaximumWindowBounds().height;
		int width = Math.min(currentWidth, MAXIMUM_WINDOW_WIDTH);
		int height = Math.min(currentHeight, MAXIMUM_WINDOW_HEIGHT);
		getFrame().setSize(width,height);
	}
	
	/**
	 * This function adds the action Listeners to the menu items
	 */
	public void addListeners(){	
		//close all button
		mainMenu.closeAll.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e){
				int size = mainWindow.openTabs.size();
				mainWindow.openTabs.clear();
			    for (int i = 0; i < size; i++) {
			    mainWindow.tabPane.remove(0);
			    	}
			}});
		//close button for particular tab
		mainMenu.close.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				int i = mainWindow.tabPane.getSelectedIndex();
			    if (i != -1) {
			    mainWindow.openTabs.remove(i);
			    mainWindow.tabPane.remove(i);
			    	}
			    }
			});
	}
	
	/**
	 * open the semester dialog
	 */
	public void openSemesterDialog() {
		semesterWindow = new SemesterWindow(this, getFrame(), controller);
		
		getFrame().setJMenuBar(null);
		
		// Adjust size and position
		getFrame().setSize(300,500);
		getFrame().setLocationRelativeTo(null);
		
		//add the window to the frame
		getFrame().add(semesterWindow);
		getFrame().setVisible(true);
	}
	
	/**
	 * close the server settings window
	 */
	public void closeServerSettingsWindow() {
		try {
			getFrame().remove(serverSettingsWindow);
		} catch (NullPointerException e) {}
	}
	
	/**
	 * close the login window
	 */
	public void closeLoginWindow() {
		try {
			getFrame().remove(loginWindow);
		} catch (NullPointerException e) {}
	}
	
	/**
	 * close the main window
	 */
	public void closeMainWindow() {
		try {
			getFrame().remove(mainWindow);
			modelHandler.removeAllListeners();
		} catch (NullPointerException e) {}
	}
	
	/**
	 * close the semester dialog
	 */
	public void closeSemesterDialog() {
		try {
			getFrame().remove(semesterWindow);
		} catch (NullPointerException e) {}
	}

	/**
	 * set the View's frame the given one
	 * @param JFrame new frame for the view
	 */
	public static void setFrame(JFrame frame) {
		View.frame = frame;
	}

	/**
	 * get the view's frame
	 * @return View's JFrame
	 */
	public static JFrame getFrame() {
		return frame;
	}
    
}
