/* AssistantPanel.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.gui.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map.Entry;

import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import tlass.project.*;
import tlass.project.gui.MainWindow;
import tlass.project.gui.SpecialComboBoxModel;
import tlass.project.model.Assistant;
import tlass.project.model.Model;
import tlass.project.model.Position;
import tlass.project.model.Skill;
import tlass.project.model.TimeBlock;


public class AssistantPanel extends ResourcePanel {
	private static final long serialVersionUID = 1L;
	private Assistant assistant;
	private Assistant assistantBackup;
	private JToolBar toolBar;
	private JPanel assistantInfoPanel;
	private GridBagConstraints constraints;
	private JTextField nameEntry, usernameEntry, limitEntry;
	private JList skillsList, timesList;
	private DefaultListModel skillsListModel, timesListModel;
	private SpecialComboBoxModel skillsModel;
	private Dimension textFieldSize = new Dimension(60,60);
	private JTree spawningTree;
	private DefaultMutableTreeNode node;
	private MainWindow mainWindow;
	private JComboBox skillBox, startHourBox, startMinBox, endHourBox, endMinBox, startMeridianBox, endMeridianBox;
	private JCheckBox activeEntry, monEntry, tuesEntry, wedEntry, thursEntry, friEntry, satEntry;

	/**
	 * Constructor for the assistant panel
	 * @param Controller for the resource panel
	 * @param Assistant assistant for the panel
	 * @param JTree
	 * @param DefaultMutableTreeNode
	 * @param MainWindow where the tab is
	 */
	public AssistantPanel(Controller c, Assistant a, JTree st, DefaultMutableTreeNode n, MainWindow mw) {
		super(c, new BorderLayout(), a); //first thing to do to send borderlayout to the jpanel
		this.assistant= a;
		this.assistantBackup= a.copy();
		this.setLayout(new BorderLayout());
		this.spawningTree= st;
		this.node= n;
		this.mainWindow= mw;

		// set up the panel which will hold the information of the selected assistant
		assistantInfoPanel = new JPanel();
		assistantInfoPanel.setLayout(new GridBagLayout());
		assistantInfoPanel.setOpaque(true);
		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.BOTH;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.weighty = 0;
		constraints.gridy = 0;
		constraints.gridx = 0;

		//Create the options toolbar
		this.toolBar= new JToolBar();
		
		//listen to see if skills change in the model
		modelHandler.addSkillsListener(new ModelListener() {

			@Override
			public void onEvent(ModelEvent event) {
				if (event.getType() == ModelEvent.ADDED){ 
					Skill newSkill = (Skill)event.getItem();
					if (!skillsModel.contains(newSkill)) {
						skillsModel.addElement(newSkill);
					}
				} else if (event.getType() == ModelEvent.REMOVED){
					Skill newSkill = (Skill)event.getItem();
					if (skillsModel.contains(newSkill)) {
						skillsModel.removeElement(newSkill);
					}
				}
			}
		
		});

		//create save and close button
		JButton saveButton= new JButton("Save and Close");
		toolBar.add(saveButton);
		saveButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {			
				mainWindow.closeTab(assistant);
			}
		});

		//create cancel changes button
		JButton cancelButton= new JButton("Cancel Changes");
		toolBar.add(cancelButton);
		cancelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				modelHandler.restoreAssistant(assistant, assistantBackup);
				mainWindow.closeTab(assistant);
			}
		});

		/*JButton removeButton= new JButton("Remove this Assistant");
		toolBar.add(removeButton);
		removeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				// Remove assistant from the Model via ModelHandler
				modelHandler.removeResource(assistant);
				
				// Remove node from tree and close tab
				DefaultTreeModel treeModel= ((DefaultTreeModel)spawningTree.getModel());			
				treeModel.removeNodeFromParent(node);
				mainWindow.closeTab(assistant);
			}
		});*/

		// Options Toolbar
		this.add(toolBar, BorderLayout.NORTH);
		toolBar.setFloatable(false);                		

		// Name
		assistantInfoPanel.add(new JLabel(" Name:  "),constraints);
		nextCol();
		nameEntry = new JTextField(assistant.getName(), 14);
		nameEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				modelHandler.setAssistantName(assistant, nameEntry.getText());
			}
		});
		assistantInfoPanel.add(nameEntry,constraints);
		nextRow();

		// Username
		assistantInfoPanel.add(new JLabel(" Username:  "),constraints);
		nextCol();
		usernameEntry = new JTextField(assistant.getUsername());
		usernameEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				modelHandler.setAssistantUsername(assistant, usernameEntry.getText());
			}
		});
		assistantInfoPanel.add(usernameEntry,constraints);		
		nextRow();
		
		// Skills
		assistantInfoPanel.add(new JLabel(" Skills:  "),constraints);
		nextCol();
		skillsListModel = new DefaultListModel();
		skillsList = new JList();
		JScrollPane skillsPane = new JScrollPane(skillsList);
		skillsList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		HashSet<Skill> skills = assistant.getSkills();
		for(Skill aSkill : skills){
			skillsListModel.addElement(aSkill);
		}
		skillsList.setModel(skillsListModel);
		assistantInfoPanel.add(skillsPane, constraints);
		nextRow();
		
		assistantInfoPanel.add(new JLabel(""),constraints);
		nextCol();
				
		//add skill editor
		JPanel skillEditorPanel= new JPanel(new FlowLayout(FlowLayout.LEADING));
		skillsModel= new SpecialComboBoxModel();
		for (Skill aSkill : model.getSkills()) {
			skillsModel.addElement(aSkill);
		}
		skillBox = new JComboBox(skillsModel);
		skillBox.setEditable(true);
		skillEditorPanel.add(skillBox);
		
		//add add skill button
		JButton skillAddButton = new JButton("Add Skill");
		skillAddButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Object selectedSkill= skillBox.getSelectedItem();
				if (selectedSkill instanceof Skill) {
					boolean added= modelHandler.addAssistantSkill(assistant, (Skill)selectedSkill);
					if (added) skillsListModel.addElement((Skill)selectedSkill);
				} else {
					String skillName= (String)selectedSkill;
					Skill newSkill = new Skill(skillName);
					boolean added= modelHandler.addAssistantSkill(assistant, newSkill);
					if (added) skillsListModel.addElement(newSkill);
				}
			}
		});
		skillEditorPanel.add(skillAddButton);
		
		//add remove skill button
		JButton skillRemoveButton = new JButton("Remove Skill");
		skillRemoveButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Object[] selectedSkills= (Object[]) skillsList.getSelectedValues();
				for (Object aSkill : selectedSkills) {
					modelHandler.removeAssistantSkill(assistant, (Skill)aSkill);
					skillsListModel.removeElement(aSkill);
				}
			}
		});
		skillEditorPanel.add(skillRemoveButton);
		
		assistantInfoPanel.add(skillEditorPanel,constraints);
		assistantInfoPanel.setMinimumSize(textFieldSize);
		nextRow();

		// Unavailable Times Panel
		// row 1
		assistantInfoPanel.add(new JLabel(" Unavailable Times:  "),constraints);
		nextCol();
		timesListModel = new DefaultListModel();
		timesList = new JList();
		JScrollPane timesPane = new JScrollPane(timesList);
		timesList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		HashSet<TimeBlock> unavailableTimes = assistant.getUnavailableTimes();
		for(TimeBlock aTime : unavailableTimes){
			timesListModel.addElement(aTime);
		}
		timesList.setModel(timesListModel);
		assistantInfoPanel.add(timesPane,constraints);
		nextRow();
		
		// row 2
		assistantInfoPanel.add(new JLabel(""),constraints);
		nextCol();
		
		//create a day editor
		JPanel daysEditorPanel= new JPanel(new FlowLayout(FlowLayout.LEADING));
		daysEditorPanel.add(new JLabel("Days: "));
		
		monEntry = new JCheckBox("Mon");
		daysEditorPanel.add(monEntry);
		tuesEntry = new JCheckBox("Tues");
		daysEditorPanel.add(tuesEntry);
		wedEntry = new JCheckBox("Wed");
		daysEditorPanel.add(wedEntry);
		thursEntry = new JCheckBox("Thurs");
		daysEditorPanel.add(thursEntry);
		friEntry = new JCheckBox("Fri");
		daysEditorPanel.add(friEntry);
		satEntry = new JCheckBox("Sat");
		daysEditorPanel.add(satEntry);
		assistantInfoPanel.add(daysEditorPanel, constraints);
		nextRow();

		// row 3
		assistantInfoPanel.add(new JLabel(""),constraints);
		nextCol();
		//assistantInfoPanel.add(new JLabel("Start Time: "),constraints);
		JPanel timesEditorPanel= new JPanel(new FlowLayout(FlowLayout.LEADING));
		timesEditorPanel.add(new JLabel("Start Time: "));
		
		Integer[] startHourTimes = {8,9,10,11,12,1,2,3,4,5,6,7,8,9};
		Integer[] minutes = {00, 05, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55};
		String[] meridian = {"am", "pm"};
		
		startHourBox = new JComboBox(startHourTimes);
		startHourBox.setEditable(false);
		timesEditorPanel.add(startHourBox);
		
		timesEditorPanel.add(new JLabel(":"));
		
		startMinBox = new JComboBox(minutes);
		startMinBox.setEditable(false);
		timesEditorPanel.add(startMinBox);
		
		startMeridianBox = new JComboBox(meridian);
		startMeridianBox.setEditable(false);
		timesEditorPanel.add(startMeridianBox);
		
		assistantInfoPanel.add(timesEditorPanel,constraints);
		assistantInfoPanel.setMinimumSize(textFieldSize);
		
		nextRow();
		
		// row 4
		assistantInfoPanel.add(new JLabel(""),constraints);
		nextCol();
		
		//create a time editor
		JPanel timesEditorPanel1= new JPanel(new FlowLayout(FlowLayout.LEADING));
		timesEditorPanel1.add(new JLabel("  End Time: "));
		
		Integer[] endHourTimes = {9,10,11,12,1,2,3,4,5,6,7,8,9,10};
		
		endHourBox = new JComboBox(endHourTimes);
		endHourBox.setEditable(false);
		timesEditorPanel1.add(endHourBox);
		
		timesEditorPanel1.add(new JLabel(":"));
		
		endMinBox = new JComboBox(minutes);
		endMinBox.setEditable(false);
		timesEditorPanel1.add(endMinBox);
		
		endMeridianBox = new JComboBox(meridian);
		endMeridianBox.setEditable(false);
		timesEditorPanel1.add(endMeridianBox);
		
		//add add time button
		JButton addTimesButton = new JButton("Add Time");
		addTimesButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int startMin= (Integer)startMinBox.getSelectedItem();
				int startHour= (Integer)startHourBox.getSelectedItem();
				int endMin= (Integer)endMinBox.getSelectedItem();
				int endHour= (Integer)endHourBox.getSelectedItem();
				
				//will be 0 if am was selected, 1 if pm was selected
				int startMeridian = startMeridianBox.getSelectedIndex();
				int endMeridian = endMeridianBox.getSelectedIndex();
				
				//change to PM if the user selected it from the dropdown
				if(startMeridian == 1){ startHour += 12; }
				if(endMeridian == 1){ endHour += 12; }
				
				if (monEntry.isSelected()) {
					TimeBlock time= new TimeBlock(TimeBlock.MONDAY, startMin, startHour, endMin, endHour);
					boolean added= modelHandler.addAssistantUnavailableTime(assistant, time);
					if (added) timesListModel.addElement(time);
				}
				if (tuesEntry.isSelected()) {
					TimeBlock time= new TimeBlock(TimeBlock.TUESDAY, startMin, startHour, endMin, endHour);
					boolean added= modelHandler.addAssistantUnavailableTime(assistant, time);
					if (added) timesListModel.addElement(time);
				}
				if (wedEntry.isSelected()) {
					TimeBlock time= new TimeBlock(TimeBlock.WEDNESDAY, startMin, startHour, endMin, endHour);
					boolean added= modelHandler.addAssistantUnavailableTime(assistant, time);
					if (added) timesListModel.addElement(time);
				}
				if (thursEntry.isSelected()) {
					TimeBlock time= new TimeBlock(TimeBlock.THURSDAY, startMin, startHour, endMin, endHour);
					boolean added= modelHandler.addAssistantUnavailableTime(assistant, time);
					if (added) timesListModel.addElement(time);
				}
				if (friEntry.isSelected()) {
					TimeBlock time= new TimeBlock(TimeBlock.FRIDAY, startMin, startHour, endMin, endHour);
					boolean added= modelHandler.addAssistantUnavailableTime(assistant, time);
					if (added) timesListModel.addElement(time);
				}
				if (satEntry.isSelected()) {
					TimeBlock time= new TimeBlock(TimeBlock.SATURDAY, startMin, startHour, endMin, endHour);
					boolean added= modelHandler.addAssistantUnavailableTime(assistant, time);
					if (added) timesListModel.addElement(time);
				}
			}
		});
		timesEditorPanel1.add(addTimesButton);
		
		//add remove time button
		JButton removeTimesButton = new JButton("Remove Time");
		removeTimesButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				for (Object selected : timesList.getSelectedValues()) {
					TimeBlock aTime= (TimeBlock) selected;
					modelHandler.removeAssistantUnavailableTime(assistant, aTime);
					timesListModel.removeElement(aTime);
				}
			}
		});
		timesEditorPanel1.add(removeTimesButton);
		
		assistantInfoPanel.add(timesEditorPanel1,constraints);
		assistantInfoPanel.setMinimumSize(textFieldSize);
		nextRow();
		
		// Student Limit Panel
		assistantInfoPanel.add(new JLabel(" Student Limit:  "),constraints);
		nextCol();
		limitEntry = new JTextField(Integer.toString(assistant.getStudentsLimit()));
		limitEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				try {
					int value= Integer.parseInt(limitEntry.getText());
					modelHandler.setAssistantLimit(assistant, value);
				} catch (Exception e) {
					// ignore bad data;
				}
			}
		});
		assistantInfoPanel.add(limitEntry,constraints);
		
		nextRow();

		// Active Field Panel
		activeEntry = new JCheckBox("Active");
		activeEntry.setSelected(assistant.isActive());
		activeEntry.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				modelHandler.setAssistantActive(assistant, activeEntry.isSelected());
			}
		});
		assistantInfoPanel.add(activeEntry,constraints);

		nextRow();

		//DUMB VERTICAL FIX...
		constraints.weighty = 1;
		assistantInfoPanel.add(new JPanel(),constraints);

		assistantInfoPanel.setBorder(BorderFactory.createLineBorder(Color.black));
		this.add(assistantInfoPanel,BorderLayout.CENTER);

	}

	//change GridBagLayout to next row
	private void nextRow() {
		constraints.gridy += 1;
		constraints.gridx = 0;
		constraints.weightx = 0;
	}

	//change GridBagLayoutto next column
	private void nextCol() {
		constraints.gridx += 1;
		constraints.weightx = 1;
	}

	// Getters
	public Assistant getAssistant(){ return assistant; }
	public JToolBar getToolBar(){ return toolBar; }

	// Setters
	public void setAssistant(Assistant a){ assistant = a; }
	public void setToolBar(JToolBar t){ toolBar = t; }
}
