/* ConstraintPanel.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.gui.panels;

import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JPanel;

import tlass.project.*;
import tlass.project.model.Constraint;


public class ConstraintPanel extends ResourcePanel {
	private static final long serialVersionUID = 1L;
	private Constraint constraint;
	private JPanel constraintPanel;
	private HashMap<JCheckBox, Constraint> constraintMap;
	
	/**
	 * Create a Constraint Panel
	 * 
	 * @param Controller which has the constraints to get
	 * @param Constraint for the constraint panel
	 */
	public ConstraintPanel(Controller con, Constraint c) {
		super(con, new GridLayout(1,0), c);
		setOpaque(true);
		this.constraint= c;
		this.controller=con;
		
		//create the constraint panel
		constraintPanel = new JPanel();
		constraintPanel.setLayout(new BoxLayout(constraintPanel, BoxLayout.PAGE_AXIS));
		constraintPanel.setOpaque(true);
		
		//Add a checkbox for each constraint
		ArrayList<Constraint> constraints = controller.getConstraints();
		constraintMap= new HashMap<JCheckBox, Constraint>();
		for(Constraint aConstraint : constraints){
			JCheckBox constraintCheckBox = new JCheckBox(aConstraint.getName(), aConstraint.isEnabled());
			constraintCheckBox.setSelected(aConstraint.isEnabled());
			constraintPanel.add(constraintCheckBox);
			
			constraintMap.put(constraintCheckBox, aConstraint);
			constraintCheckBox.addItemListener(new ItemListener() {
				@Override
				public void itemStateChanged(ItemEvent e) {
					boolean isSelected= e.getStateChange() == ItemEvent.SELECTED;
					JCheckBox source= (JCheckBox) e.getSource();
					constraintMap.get(source).setEnabled(isSelected);
				}
			});
		}
		//add the constraint panel to the resource panel
		this.add(constraintPanel);
	}
	
	/**
	 * Getter for constraint
	 * @return
	 */
	public Constraint getConstraint(){ return constraint; }
	
	/**
	 * Setter for constraint
	 * @param c
	 */
	public void setConstraint(Constraint c){ constraint = c; }
}
