package tlass.project.gui.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.tree.DefaultMutableTreeNode;

import tlass.project.Controller;
import tlass.project.ModelEvent;
import tlass.project.ModelListener;
import tlass.project.gui.MainWindow;
import tlass.project.gui.panels.ResourcePanel;
import tlass.project.model.Assistant;
import tlass.project.model.Instructor;


public class InstructorPanel extends ResourcePanel {

	private static final long serialVersionUID = 1L;
	private Instructor instructor;
	private Instructor instructorBackup;
	private JToolBar toolBar;
	private JPanel instructorInfoPanel;
	private GridBagConstraints constraints;
	private JTextField nameEntry, usernameEntry;
	private JList forbidList;
	private DefaultListModel forbidListModel;
	private DefaultComboBoxModel assistantModel;
	private MainWindow mainWindow;
	@SuppressWarnings("unused")
	private JTree spawningTree;
	@SuppressWarnings("unused")
	private DefaultMutableTreeNode node;
	private JComboBox assistantBox;
	private Dimension textFieldSize = new Dimension(60,60);
	
	/**
	 * Constructor for Instructor Panels
	 * @param Controller for Resource Panel
	 * @param Instructor for the panel
	 * @param JTree spawning tree, not used in this version
	 * @param DefaultMutableTreeNode, not used in this version
	 * @param MainWindow the window where the panel is
	 */
	public InstructorPanel(Controller c, Instructor i, JTree st, DefaultMutableTreeNode n, MainWindow mw) {
		super(c, new BorderLayout(), i); //first thing to do to send borderlayout to the jpanel
		this.instructor = i;
		this.instructorBackup = i;
		this.setLayout(new BorderLayout());
		this.mainWindow = mw;
		this.spawningTree = st;
		this.node = n;
		
		// set up the panel which will hold the information of the selected instructor
		instructorInfoPanel = new JPanel();
		instructorInfoPanel.setLayout(new GridBagLayout());
		instructorInfoPanel.setOpaque(true);
		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.BOTH;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.weighty = 0;
		constraints.gridy = 0;
	    constraints.gridx = 0;
		
		modelHandler.addAssistantsListener(new ModelListener() {

			@Override
			public void onEvent(ModelEvent event) {
				// TODO Auto-generated method stub
				if (event.getType() == ModelEvent.ADDED){ 
					Assistant a = (Assistant)event.getItem();
					assistantModel.addElement(a);
				} else if (event.getType() == ModelEvent.REMOVED){
					Assistant a = (Assistant)event.getItem();
					assistantModel.removeElement(a);
					forbidListModel.removeElement(a);
				}	
			}
			
		});
		
		//Create the options toolbar
        this.toolBar= new JToolBar();
        
        //create the save and close button
        JButton addRowButton= new JButton("Save and Close");
        toolBar.add(addRowButton);
        addRowButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				mainWindow.closeTab(instructor);
			}
        });
        
        //create cancel changes button
        JButton saveButton= new JButton("Cancel Changes");
        toolBar.add(saveButton);
        saveButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				modelHandler.restoreInstructor(instructor, instructorBackup);
				mainWindow.closeTab(instructor);
			}
        });
        
        /*JButton reloadButton= new JButton("Remove this Instructor");
        toolBar.add(reloadButton);
        reloadButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// Remove instructor from the Model via ModelHandler
				modelHandler.removeResource(instructor);
				
				// Remove node from tree and close tab
				DefaultTreeModel treeModel= ((DefaultTreeModel)spawningTree.getModel());			
				treeModel.removeNodeFromParent(node);
				mainWindow.closeTab(instructor);				
			}
        });*/
        
        this.add(toolBar, BorderLayout.NORTH);	
        toolBar.setFloatable(false);
		
        // Name
        instructorInfoPanel.add(new JLabel(" Name:  "), constraints);
		nextCol();
        nameEntry = new JTextField(instructor.getName());
        nameEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				modelHandler.setInstructorName(instructor, nameEntry.getText());
			}
		});
        instructorInfoPanel.add(nameEntry, constraints);
        nextRow();        
        
		// Username
        instructorInfoPanel.add(new JLabel(" Username:  "),constraints);
		nextCol();
        usernameEntry = new JTextField(instructor.getId());
        usernameEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				modelHandler.setInstructorUsername(instructor, usernameEntry.getText());
			}
		});
		instructorInfoPanel.add(usernameEntry,constraints);
		nextRow();
		
		// Forbidden Assistants 
		instructorInfoPanel.add(new JLabel(" Forbidden Assistants:  "),constraints);
		nextCol();
		forbidListModel = new DefaultListModel();
		forbidList = new JList();
		JScrollPane forbidPane = new JScrollPane(forbidList);
		forbidList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		ArrayList<Assistant> forbiddenAssistants = instructor.getForbiddenAssistants();
		for(Assistant anAssistant : forbiddenAssistants){
			forbidListModel.addElement(anAssistant);
		}
		forbidList.setModel(forbidListModel);
		instructorInfoPanel.add(forbidPane, constraints);
		nextRow();
	
		// Forbidden Assistants Toolbar
		instructorInfoPanel.add(new JLabel(""),constraints);
		nextCol();
		JPanel assistantEditorPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
		
		assistantModel = new DefaultComboBoxModel();
		ArrayList<Assistant> assistants = model.getAssistants();
		for(Assistant anAssistant : assistants){
			assistantModel.addElement(anAssistant);
		}
		assistantBox = new JComboBox(assistantModel);
		assistantEditorPanel.add(assistantBox);
		
		JButton assistantAddButton = new JButton("Add");
		assistantAddButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0){
				Object selectedAssistant = assistantBox.getSelectedItem();
				if(!forbidListModel.contains(selectedAssistant)){
						forbidListModel.addElement(selectedAssistant);
						assistantBox.removeItem(assistantBox.getSelectedItem());
					}
			}
		
		});
		assistantEditorPanel.add(assistantAddButton);
		
		//remove button
		JButton assistantRemoveButton = new JButton("Remove this Assistant");
		assistantRemoveButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0){
				Object selectedAssistant = forbidList.getSelectedValue();
				if(selectedAssistant instanceof Assistant){
					forbidListModel.removeElement(selectedAssistant);
					modelHandler.removeResource(selectedAssistant);
					assistantBox.addItem(selectedAssistant);
				}
			}
		});
		assistantEditorPanel.add(assistantRemoveButton);
		
		instructorInfoPanel.add(assistantEditorPanel,constraints);
		instructorInfoPanel.setMinimumSize(textFieldSize);
		nextRow();
		
		//DUMB VERTICAL FIX...
		constraints.weighty = 1;
		instructorInfoPanel.add(new JPanel(),constraints);
		
		instructorInfoPanel.setBorder(BorderFactory.createLineBorder(Color.black));
		this.add(instructorInfoPanel,BorderLayout.CENTER);
	
	}
	
	/**
	 * move to the next row in our GridBagLayout
	 */
	private void nextRow() {
		constraints.gridy += 1;
		constraints.gridx = 0;
		constraints.weightx = 0;
	}
	
	/**
	 * move to the next column in our GridBagLayout
	 */
	private void nextCol() {
		constraints.gridx += 1;
		constraints.weightx = 1;
	}

	/**
	 * set the instructor for the panel
	 * @param Instructor
	 */
	public void setInstructor(Instructor i) {
		instructor = i;
	}

	/**
	 * get the instructor of the panel
	 * @return Instructor
	 */
	public Instructor getInstructor() {
		return instructor;
	}
}
