package tlass.project.gui.panels;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import tlass.project.Controller;
import tlass.project.ModelHandler;
import tlass.project.Pair;
import tlass.project.WebformHandler;
import tlass.project.model.Assistant;
import tlass.project.model.Model;
import tlass.project.model.Position;
import tlass.project.model.TimeBlock;

public class RestrictionDialog extends JDialog {
	private Controller controller;
	private WebformHandler webformHandler;
	private ModelHandler modelHandler;
	private JFrame owner;
	private Model model;
	private RestrictionDialog self; // Used to reference this inside of an anonymous inner
						// class
	private Restriction currentRestriction;
	private Restriction backupRestriction;
	private Object backupData;
	private ArrayList<Assistant> forbiddens;
	private Assistant required;
	private Position position;
	private JComboBox cb;

	private JPanel root;
	private JPanel masterPanel;
	private DefaultComboBoxModel assistantBoxModel;
	private JButton addForbidden;
	private JButton removeForbidden;

	/**
	 * Constructor for RestrictionDialog. 
	 * Forces it to pop up and handle the data itself.
	 * @param o
	 * @param c
	 * @param p
	 * @param cr
	 * @param data
	 */
	@SuppressWarnings("unchecked")
	public RestrictionDialog(JFrame o, Controller c, Position p, Restriction cr, Object data) {
		super(o, true);
		this.controller = c;
		this.owner = o;
		this.self = this;
		this.model = c.getModel();
		this.modelHandler = c.getModelHandler();
		webformHandler = c.getWebformHandler();
		this.position= p;
		this.currentRestriction= cr;
		this.backupRestriction= cr;
		required= null;
		forbiddens= new ArrayList<Assistant>();
		if (cr == Restriction.MULTIPLE) {
			forbiddens= (ArrayList<Assistant>)data;
			backupData= (ArrayList<Assistant>)data;
		} else if (cr == Restriction.SINGLE){
			required= (Assistant)data;
			backupData= (Assistant)data;
		}
		
		root = new JPanel(new BorderLayout());
		masterPanel= new JPanel(new BorderLayout());
		
		String[] comboBoxItems= { "Require", "Forbid", "Unrestricted"};
		cb = new JComboBox(comboBoxItems);
		cb.setSelectedIndex(currentRestriction.getId());
		cb.setEditable(false);
		cb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
			    int panel= cb.getSelectedIndex();
			    if (panel==0) {
			    	currentRestriction= Restriction.SINGLE;
			    } else if (panel==1) {
			    	currentRestriction= Restriction.MULTIPLE;
			    } else {
			    	currentRestriction= Restriction.NONE;
			    }
			    changeRestrictions();
			}
		});
		
		assistantBoxModel = new DefaultComboBoxModel();
		for (Assistant a : model.getAssistants()) {
			assistantBoxModel.addElement(a);
		}
		if(required!=null) {
			assistantBoxModel.setSelectedItem(required);
		}
		assistantBox = new JComboBox(assistantBoxModel);
		
		forbiddenModel = new DefaultListModel();
		forbiddenList = new JList(forbiddenModel);
//		JScrollPane openTaPositionsPane = new JScrollPane(forbiddenList);
		forbiddenList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		for (Assistant a : forbiddens) {
			forbiddenModel.addElement(a);
		}
		
		addForbidden = new JButton("Add");
		addForbidden.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Assistant selected= (Assistant) assistantBox.getSelectedItem();
				if (!forbiddens.contains(selected)) {
					forbiddens.add(selected);
					forbiddenModel.addElement(selected);
					pack();
				}
			}
		});
		
		removeForbidden = new JButton("Remove");
		removeForbidden.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				for (Object a : forbiddenList.getSelectedValues()) {
					Assistant selected = (Assistant) a;
					forbiddens.remove(selected);
					forbiddenModel.removeElement(selected);
					pack();
				}
			}
		});
		
		changeRestrictions();
		
		root.add(cb, BorderLayout.PAGE_START);
		
		root.add(masterPanel, BorderLayout.CENTER);

		JPanel toolbar= new JPanel();
		toolbar.setLayout(new FlowLayout(FlowLayout.LEADING));
		toolbar.setOpaque(true);
		JButton cancel= new JButton("Cancel");
		cancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				modelHandler.setPositionRestriction(position, backupRestriction, backupData);
				self.dispose();
			}
		});
		toolbar.add(cancel);
		JButton okay= new JButton("Okay");
		okay.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (currentRestriction == Restriction.SINGLE) {
					modelHandler.setPositionRestriction(position, currentRestriction, (Assistant) assistantBox.getSelectedItem());
				} else if(currentRestriction != backupRestriction) {
					if (currentRestriction == Restriction.MULTIPLE) {
					modelHandler.setPositionRestriction(position, currentRestriction, forbiddens);
					} else {
					modelHandler.setPositionRestriction(position, currentRestriction, null);
					}
				}
				self.dispose();
			}
		});
		toolbar.add(okay);
		root.add(toolbar, BorderLayout.PAGE_END);
		
		getContentPane().add(root);
		
		// Display all the users and times
		setSize(500, 600);
		setLocationRelativeTo(null);
		pack();
		setVisible(true);
	}

	/**
	 * Changes which dialog elements are currently visible based on
	 * what restriction has been chosen
	 */
	private void changeRestrictions() {
		masterPanel.removeAll();
		if (currentRestriction == Restriction.SINGLE) {
	    	masterPanel.add(assistantBox, BorderLayout.WEST);
	    } else if (currentRestriction == Restriction.MULTIPLE) {
	    	masterPanel.add(assistantBox, BorderLayout.WEST);
	    	masterPanel.add(forbiddenList, BorderLayout.PAGE_START);
	    	masterPanel.add(addForbidden, BorderLayout.CENTER);
	    	masterPanel.add(removeForbidden, BorderLayout.EAST);
	    }
		pack();
	}
	
	private DefaultListModel forbiddenModel;
	private JList forbiddenList;
	private JComboBox assistantBox; 

}
