/* SectionPanel.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.gui.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map.Entry;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.MaskFormatter;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import tlass.project.*;
import tlass.project.gui.MainWindow;
import tlass.project.gui.SpecialComboBoxModel;
import tlass.project.model.Assistant;
import tlass.project.model.Instructor;
import tlass.project.model.Position;
import tlass.project.model.Section;
import tlass.project.model.Skill;
import tlass.project.model.TimeBlock;


public class SectionPanel extends ResourcePanel {
	private Section section;
	private Section sectionBackup;
	private JToolBar toolBar;
	private JPanel sectionInfoPanel, startTimePanel, endTimePanel, dayPanel, taButtonPanel, laButtonPanel, forbidPanel, requirePanel;
	private GridBagConstraints constraints;
	private JList taPositionsList, laPositionsList;
	private DefaultListModel taPositionsListModel, laPositionsListModel;
	private Dimension textFieldSize = new Dimension(60,60);
	private JFormattedTextField idEntry, locationEntry, enrolledMaxEntry;
	final String validChars  = "abcdefghijklmnopqrstuvwxyABCDEFGHIJKLMNOPQRSTUVWXYZ .";
	final String validTimeChars = "1234567890: apm";
	JCheckBox activeEntry, assistantsPresentEntry;
	private DefaultListModel skillsListModel;
	private SpecialComboBoxModel skillsModel;
	private JList skillsList;
	private JComboBox skillBox, instructorBox;
	private MainWindow mainWindow;
	private MaskFormatter format, tFormat;
	private JCheckBox mondayCheck, tuesdayCheck, wednesdayCheck, thursdayCheck, fridayCheck, saturdayCheck;
	private String[] hours = {"1","2","3","4","5","6","7","8","9","10","11","12"};
	private String[] minuites = {"00", "05", "10", "15", "20", "25", "30", "35", "40", "45", "50", "55"};
	private String[] meridian = {"AM", "PM"};
	private JFrame frame;

	private JComboBox startHoursList = new JComboBox(hours);
	private JComboBox startMinutesList = new JComboBox(minuites);
	private JComboBox startMeridianList = new JComboBox(meridian);

	private JComboBox endHoursList = new JComboBox(hours);
	private JComboBox endMinutesList = new JComboBox(minuites);
	private JComboBox endMeridianList = new JComboBox(meridian);
	ArrayList<JCheckBox> daysOfTheWeek = new ArrayList<JCheckBox>();
	private JPanel skillButtonPanel;

	public SectionPanel(Controller c, Section s, JTree st, DefaultMutableTreeNode n, MainWindow mw) {
		super(c, new BorderLayout(), s); //first thing to do to send borderlayout to the jpanel
		this.section= s;
		this.sectionBackup= s;
		this.mainWindow= mw;
		this.setLayout(new BorderLayout());
		frame = new JFrame();

		// set up the panels which will hold the information of the selected section		
		sectionInfoPanel = new JPanel();
		sectionInfoPanel.setLayout(new GridBagLayout());
		sectionInfoPanel.setOpaque(true);

		modelHandler.addSkillsListener(new ModelListener() {

			@Override
			public void onEvent(ModelEvent event) {
				if (event.getType() == ModelEvent.ADDED){ 
					Skill newSkill = (Skill)event.getItem();
					if (!skillsModel.contains(newSkill)) {
						skillsModel.addElement(newSkill);
					}
				} else if (event.getType() == ModelEvent.REMOVED){
					Skill newSkill = (Skill)event.getItem();
					if (skillsModel.contains(newSkill)) {
						skillsModel.removeElement(newSkill);
					}
				}
			}

		});

		startTimePanel = new JPanel();
		startTimePanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		startTimePanel.setOpaque(true);

		endTimePanel = new JPanel();
		endTimePanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		endTimePanel.setOpaque(true);

		dayPanel = new JPanel();
		dayPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		dayPanel.setOpaque(true);

		taButtonPanel = new JPanel();
		taButtonPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		taButtonPanel.setOpaque(true);

		laButtonPanel = new JPanel();
		laButtonPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		laButtonPanel.setOpaque(true);

		forbidPanel = new JPanel();
		forbidPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		forbidPanel.setOpaque(true);

		requirePanel = new JPanel();
		requirePanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		requirePanel.setOpaque(true);

		// initialize the JCheckBoxes
		mondayCheck = new JCheckBox("Mon");
		tuesdayCheck = new JCheckBox("Tue");
		wednesdayCheck = new JCheckBox("Wed");
		thursdayCheck = new JCheckBox("Thu");
		fridayCheck = new JCheckBox("Fri");
		saturdayCheck = new JCheckBox("Sat");
		mondayCheck.setSelected(section.hasDay(TimeBlock.MONDAY));
		tuesdayCheck.setSelected(section.hasDay(TimeBlock.TUESDAY));
		wednesdayCheck.setSelected(section.hasDay(TimeBlock.WEDNESDAY));
		thursdayCheck.setSelected(section.hasDay(TimeBlock.THURSDAY));
		fridayCheck.setSelected(section.hasDay(TimeBlock.FRIDAY));
		saturdayCheck.setSelected(section.hasDay(TimeBlock.SATURDAY));
		mondayCheck.addItemListener(new ItemListenerWithDay(TimeBlock.MONDAY));
		tuesdayCheck.addItemListener(new ItemListenerWithDay(TimeBlock.TUESDAY));
		wednesdayCheck.addItemListener(new ItemListenerWithDay(TimeBlock.WEDNESDAY));
		thursdayCheck.addItemListener(new ItemListenerWithDay(TimeBlock.THURSDAY));
		fridayCheck.addItemListener(new ItemListenerWithDay(TimeBlock.FRIDAY));
		saturdayCheck.addItemListener(new ItemListenerWithDay(TimeBlock.SATURDAY));

		daysOfTheWeek.add(mondayCheck);
		daysOfTheWeek.add(tuesdayCheck);
		daysOfTheWeek.add(wednesdayCheck);
		daysOfTheWeek.add(thursdayCheck);
		daysOfTheWeek.add(fridayCheck);
		daysOfTheWeek.add(saturdayCheck);

		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.BOTH;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.weighty = 0;
		constraints.gridy = 0;
		constraints.gridx = 0;

		try {
			format = new MaskFormatter("************************");
			tFormat = new MaskFormatter("********");
		} catch (ParseException e) {
			e.printStackTrace();
		}
		format.setValidCharacters(validChars);
		tFormat.setValidCharacters(validTimeChars);

		//Create the options toolbar
		this.toolBar= new JToolBar();

		JButton addRowButton= new JButton("Save and Close");
		toolBar.add(addRowButton);
		addRowButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// closes the tab
				int index= mainWindow.tabPane.getSelectedIndex();
				mainWindow.closeTab(index);
			}
		});

		// Creates cancel button which will cancel the changes.
		JButton cancelButton= new JButton("Cancel Changes");
		toolBar.add(cancelButton);
		// Adds an actionListener to cancel button.
		cancelButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				modelHandler.restoreSection(section, sectionBackup);
				mainWindow.closeTab(section);
			}
		});

		// Creates a button that will add a ta position
		// and adds an actionListner
		JButton addSectionTa = new JButton("Add TA Position");
		addSectionTa.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Position p= modelHandler.addTAPosition(section);
				taPositionsListModel.addElement(p);				
			}
		});

		// Creates a button that will remvoe a ta position
		// and adds an actionListener
		JButton removeSectionTa = new JButton("Remove TA Position");
		removeSectionTa.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Position position= (Position) taPositionsListModel.lastElement();
				modelHandler.removeTAPosition(position);
				taPositionsListModel.removeElement(position);
			}
		});

		// Creates a button that will add an la position
		// and adds an actionListener
		JButton addSectionLa = new JButton("Add LA Position");
		addSectionLa.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Position p= modelHandler.addLAPosition(section);
				laPositionsListModel.addElement(p);
			}
		});

		// Creates a button that will remove an la position 
		JButton removeSectionLa = new JButton("Remove LA Position");
		removeSectionLa.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Position position= (Position) laPositionsListModel.lastElement();
				modelHandler.removeLAPosition(position);
				laPositionsListModel.removeElement(position);
			}
		});


		//Creates a button that will bring a dialog that will edit restrictions for Tas
		// and adds an actionListener
		JButton editTaRestrictions = new JButton("Edit Restrictions");
		editTaRestrictions.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0){
				Position position = (Position) taPositionsList.getSelectedValue();
				if (position == null) return;
				askRestrictions(position, position.getRestriction(), position.getRestrictionValue());
			}
		});

		//Creates a button that will bring a dialog that will edit restrictins for Las
		JButton editLaRestrictions = new JButton("Edit Restrictions");
		editLaRestrictions.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0){
				Position position = (Position) laPositionsList.getSelectedValue();
				if (position == null) return;
				askRestrictions(position, position.getRestriction(), position.getRestrictionValue());
			}
		});

		this.add(toolBar, BorderLayout.NORTH);	
		toolBar.setFloatable(false);

		// Id
		sectionInfoPanel.add(new JLabel(" ID:  "),constraints);
		nextCol();
		idEntry = new JFormattedTextField(createFormatter("******************"));
		idEntry.setValue(section.getId());
		idEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				modelHandler.setSectionId(section, idEntry.getText());
			}
		});
		sectionInfoPanel.add(idEntry,constraints);
		nextRow();

		// Location 
		sectionInfoPanel.add(new JLabel(" Location:  "),constraints);
		nextCol();
		locationEntry = new JFormattedTextField(format);
		locationEntry.setValue(section.getLocation());
		locationEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				modelHandler.setSectionLocation(section, locationEntry.getText());
			}
		});
		sectionInfoPanel.add(locationEntry,constraints);
		nextRow();

		// Instructors 
		sectionInfoPanel.add(new JLabel(" Instructor:  "),constraints);
		nextCol();
		instructorBox = new JComboBox();
		for(Instructor instruct: model.getInstructors()){
			instructorBox.addItem(instruct);
		}
		instructorBox.setSelectedItem(section.getInstructor());
		instructorBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				modelHandler.setSectionInstructor(section, (Instructor) instructorBox.getSelectedItem());
			}
		});
		sectionInfoPanel.add(instructorBox, constraints);
		nextRow();

		// Day
		nextCol();
		dayPanel.add(new JLabel("Days:   "), constraints);
		dayPanel.add(mondayCheck, constraints);
		dayPanel.add(tuesdayCheck, constraints);
		dayPanel.add(wednesdayCheck, constraints);
		dayPanel.add(thursdayCheck, constraints);
		dayPanel.add(fridayCheck, constraints);
		dayPanel.add(saturdayCheck, constraints);
		sectionInfoPanel.add(dayPanel, constraints);
		nextRow();

		// Time
		ActionListener timeChanged = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int startMinute= Integer.parseInt((String) startMinutesList.getSelectedItem());
				int endMinute= Integer.parseInt((String)endMinutesList.getSelectedItem());
				int startHour= Integer.parseInt((String)startHoursList.getSelectedItem());
				int endHour= Integer.parseInt((String)endHoursList.getSelectedItem());
				String startMeridian= (String)startMeridianList.getSelectedItem();
				String endMeridian= (String)endMeridianList.getSelectedItem();
				int start= TimeBlock.makeTime(startMinute, startHour, startMeridian);
				int end= TimeBlock.makeTime(endMinute, endHour, endMeridian);
				modelHandler.setSectionTime(section, start, end);
			}
		};

		sectionInfoPanel.add(new JLabel(" Start Time:  "),constraints);
		nextCol();
		// set start time
		startHoursList.setSelectedItem(section.getCurrentTime().getStartHourString());
		startMinutesList.setSelectedItem(section.getCurrentTime().getStartMinuteString());
		startMeridianList.setSelectedItem(section.getCurrentTime().getStartMerdian());
		startTimePanel.add(startHoursList, constraints);
		startTimePanel.add(startMinutesList, constraints);
		startTimePanel.add(startMeridianList, constraints);
		sectionInfoPanel.add(startTimePanel, constraints);
		nextRow();

		sectionInfoPanel.add(new JLabel(" End Time:  "),constraints);
		nextCol();
		// set the end time.
		endHoursList.setSelectedItem(section.getCurrentTime().getEndHourString());
		endMinutesList.setSelectedItem(section.getCurrentTime().getEndMinuteString());
		endMeridianList.setSelectedItem(section.getCurrentTime().getEndMerdian());
		endTimePanel.add(endHoursList, constraints);
		endTimePanel.add(endMinutesList, constraints);
		endTimePanel.add(endMeridianList, constraints);
		sectionInfoPanel.add(endTimePanel, constraints);
		nextRow();

		startMinutesList.addActionListener(timeChanged);
		endMinutesList.addActionListener(timeChanged);
		startHoursList.addActionListener(timeChanged);
		endHoursList.addActionListener(timeChanged);
		startMeridianList.addActionListener(timeChanged);
		endMeridianList.addActionListener(timeChanged);

		// Max Enrolled
		sectionInfoPanel.add(new JLabel(" Maximum Enrollment:  "),constraints);
		nextCol();
		enrolledMaxEntry = new JFormattedTextField(createFormatter("####"));
		enrolledMaxEntry.setValue(Integer.toString(section.getEnrolledMax()));
		enrolledMaxEntry.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent arg0) {}
			@Override
			public void insertUpdate(DocumentEvent arg0) {update();}
			@Override
			public void removeUpdate(DocumentEvent arg0) {update();}
			public void update() {
				modelHandler.setSectionEnrolledMax(section, enrolledMaxEntry.getText());
			}
		});
		sectionInfoPanel.add(enrolledMaxEntry,constraints);
		nextRow();

		// Skills Needed
		sectionInfoPanel.add(new JLabel(" Skills Needed:  "),constraints);
		nextCol();
		JPanel skillEditorPanel= new JPanel(new BorderLayout());
		skillsListModel = new DefaultListModel();
		skillsList = new JList();
		JScrollPane skillsPane = new JScrollPane(skillsList);
		skillsList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		HashSet<Skill> skills = section.getSkillsNeeded();
		for(Skill aSkill : skills){
			skillsListModel.addElement(aSkill);
		}
		skillsList.setModel(skillsListModel);
		skillsPane.setMinimumSize(textFieldSize);
		skillEditorPanel.add(skillsPane, BorderLayout.PAGE_START);

		sectionInfoPanel.add(skillEditorPanel,constraints);
		sectionInfoPanel.setMinimumSize(textFieldSize);
		nextRow();

		// Edit Skills Toolbar
		sectionInfoPanel.add(new JLabel(" "),constraints);
		nextCol();
		JPanel skillEditorPanel2= new JPanel(new BorderLayout());

		skillButtonPanel = new JPanel();
		skillButtonPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		skillButtonPanel.setOpaque(true);

		skillsModel= new SpecialComboBoxModel();
		for (Skill aSkill : model.getSkills()) {
			skillsModel.addElement(aSkill);
		}
		skillBox = new JComboBox(skillsModel);
		skillBox.setEditable(true);
		skillButtonPanel.add(skillBox);

		// Creates the button responsible for adding a new skill.
		JButton skillAddButton2 = new JButton("Add");
		// Adds an actionlistener to skillAddButton2
		skillAddButton2.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Object selectedSkill= skillBox.getSelectedItem();
				if (selectedSkill instanceof Skill) {
					boolean added= modelHandler.addSectionSkill(section, (Skill)selectedSkill);
					if (added) skillsListModel.addElement((Skill)selectedSkill);
				} else {
					String skillName= (String)selectedSkill;
					Skill newSkill = new Skill(skillName);
					boolean added= modelHandler.addSectionSkill(section, newSkill);
					if (added) skillsListModel.addElement(newSkill);
				}
			}
		});

		// Adds the skillAddButton2 to the skill editor panel.
		skillButtonPanel.add(skillAddButton2, BorderLayout.CENTER);

		// Creates the button responsible for removing skills
		JButton skillRemoveButton2 = new JButton("Remove");
		// Adds an action listener to the remove button
		// this removes the selected skills from skillsListModel
		skillRemoveButton2.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Object[] selectedSkills= (Object[]) skillsList.getSelectedValues();
				for (Object aSkill : selectedSkills) {
					modelHandler.removeSectionSkill(section, (Skill)aSkill);
					skillsListModel.removeElement((Skill)aSkill);
				}
			}
		});
		// Adds the remove skill button to the skill editor panel.
		skillButtonPanel.add(skillRemoveButton2, BorderLayout.EAST);

		skillEditorPanel2.add(skillButtonPanel, BorderLayout.CENTER);

		sectionInfoPanel.add(skillEditorPanel2,constraints);
		sectionInfoPanel.setMinimumSize(textFieldSize);
		nextRow();

		// Open TA Positions
		sectionInfoPanel.add(new JLabel("TA Positions:  "),constraints);
		nextCol();
		taPositionsListModel = new DefaultListModel();
		taPositionsList = new JList(taPositionsListModel);
		taPositionsList.setCellRenderer(new PositionCellRender());
		JScrollPane openTaPositionsPane = new JScrollPane(taPositionsList);
		taPositionsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		for (Position p : section.getTAs()){
			taPositionsListModel.addElement(p);
		}

		openTaPositionsPane.setMinimumSize(textFieldSize);
		sectionInfoPanel.add(openTaPositionsPane,constraints);

		nextRow();
		nextCol();
		// add add/remove buttons
		taButtonPanel.add(addSectionTa, constraints);
		taButtonPanel.add(removeSectionTa, constraints);
		taButtonPanel.add(editTaRestrictions, constraints);
		sectionInfoPanel.add(taButtonPanel, constraints);
		nextRow();

		// Open LA Positions
		sectionInfoPanel.add(new JLabel("LA Positions:  "),constraints);
		nextCol();
		laPositionsListModel = new DefaultListModel();
		laPositionsList = new JList(laPositionsListModel);
		laPositionsList.setCellRenderer(new PositionCellRender());
		JScrollPane openLaPositionsPane = new JScrollPane(laPositionsList);
		laPositionsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		for (Position p : section.getLAs()){
			laPositionsListModel.addElement(p);
		}
		openLaPositionsPane.setMinimumSize(textFieldSize);
		sectionInfoPanel.add(openLaPositionsPane,constraints);

		nextRow();
		nextCol();
		// Adds buttons to the bottom
		laButtonPanel.add(addSectionLa, constraints);
		laButtonPanel.add(removeSectionLa, constraints);
		laButtonPanel.add(editLaRestrictions, constraints);
		sectionInfoPanel.add(laButtonPanel, constraints);
		nextRow();
		// Active
		//		activeEntry = new JCheckBox("Active");
		//		activeEntry.setSelected(section.isActive());
		//		sectionInfoPanel.add(activeEntry,constraints);
		nextRow();



		//DUMB VERTICAL FIX...
		constraints.weighty = 1;
		sectionInfoPanel.add(new JPanel(),constraints);

		sectionInfoPanel.setBorder(BorderFactory.createLineBorder(Color.black));
		this.add(sectionInfoPanel,BorderLayout.CENTER);

	}

	private class ItemListenerWithDay implements ItemListener {
		private int day;
		public ItemListenerWithDay(int day) {
			this.day= day;
		}
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				modelHandler.removeSectionTimeDay(section, day);
			} else {
				int startMinute= Integer.parseInt((String) startMinutesList.getSelectedItem());
				int endMinute= Integer.parseInt((String)endMinutesList.getSelectedItem());
				int startHour= Integer.parseInt((String)startHoursList.getSelectedItem());
				int endHour= Integer.parseInt((String)endHoursList.getSelectedItem());
				String startMeridian= (String)startMeridianList.getSelectedItem();
				String endMeridian= (String)endMeridianList.getSelectedItem();
				int start= TimeBlock.makeTime(startMinute, startHour, startMeridian);
				int end= TimeBlock.makeTime(endMinute, endHour, endMeridian);
				modelHandler.addSectionTime(section, day, start, end);
			}
		}
	}

	/**
	 * Retrieves the restrictions for a position and updates the GUI
	 * @param currentType
	 * @param data
	 * @return
	 */
	public Pair<Restriction, Object> askRestrictions(Position p, Restriction currentType, Object data) {
		new RestrictionDialog(frame, controller, p, currentType, data);
		taPositionsList.repaint();
		laPositionsList.repaint();
		return null;
	}

	private class PositionCellRender  implements ListCellRenderer {

		protected DefaultListCellRenderer defaultRenderer = new DefaultListCellRenderer();

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			Position p = (Position) value;
			JLabel renderer= (JLabel) defaultRenderer.getListCellRendererComponent(list, value, index,
					isSelected, cellHasFocus);
			if (p.getRestriction() == Restriction.NONE) { 
				renderer.setText("#"+p.getOrder()+": No restrictions");
			} else if (p.getRestriction() == Restriction.SINGLE) { 
				renderer.setText("#"+p.getOrder()+": Requires "+p.getRestrictionValue());
			} else {
				renderer.setText("#"+p.getOrder()+": Forbids "+p.getRestrictionValue());
			}

			//renderer.setBorder(cellHasFocus ? focusBorder : noFocusBorder);

			return renderer;
		}
	}
	/**
	 * A convenience method for creating a MaskFormatter.
	 * @param s
	 * @return
	 */
	protected MaskFormatter createFormatter(String s) {
		MaskFormatter formatter = null;
		try {
			formatter = new MaskFormatter(s);
		} catch (java.text.ParseException exc) {
			System.err.println("formatter is bad: " + exc.getMessage());
			System.exit(-1);
		}
		return formatter;
	}

	/**
	 * Increments the the y, moving to the next row.
	 */
	private void nextRow() {
		constraints.gridy += 1;
		constraints.gridx = 0;
		constraints.weightx = 0;
	}

	/**
	 * Increments the x, moving to the next column.
	 */
	private void nextCol() {
		constraints.gridx += 1;
		constraints.weightx = 1;
	}

	// Getters
	public Section getSection(){ return section; }

	// Setters
	public void setSection(Section s){ section = s; }



}


