/* Assistant.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;


public class Assistant implements Serializable {
	
	//variables for new system design
	private String username;
	private String name;
	private boolean active;
	private HashSet<Skill> skills;
	private HashSet<TimeBlock> unavailableTimes;
	private int studentsLimit;
	
	/**
	 * Creates a new assistant
	 */
	public Assistant() {
		this.skills= new HashSet<Skill>();
		this.unavailableTimes= new HashSet<TimeBlock>();
		this.name= "New Assistant";
		this.username= "";
		this.active= true;
		this.studentsLimit= 30;
	}
	
	/**
	 * Creates an exact copy of an assistant
	 * @return
	 */
	public Assistant copy() {
		Assistant clone = new Assistant();
		clone.setActive(active);
		clone.setName(name);
		clone.setSkills((HashSet<Skill>) skills.clone());
		clone.setStudentsLimit(studentsLimit);
		clone.setUnavailableTimes((HashSet<TimeBlock>) unavailableTimes.clone());
		clone.setUsername(username);
		return clone;
	}
	
	public void clean() {
		//Nothing to clean
	}
	
	/**
	 * Restores an assistant with all the data from a previous version
	 * @param backup
	 */
	public void restore(Assistant backup) {
		active= backup.isActive();
		name= backup.getName();
		skills= backup.getSkills();
		studentsLimit= backup.getStudentsLimit();
		unavailableTimes= backup.getUnavailableTimes();
		username= backup.getUsername();
	}
	
	/**
	 * prints out the name of an assitant when referenced just as Assistant
	 */
	public String toString() {
		return name;
	}

	/**
	 * sets the username for this assistant
	 * @param username
	 */
	public void setUsername(String username) {
		this.username = username;
	}

	/**
	 * returns the username
	 * @return
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * sets the assistant's name
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * returns the assistant's name
	 * @return
	 */
	public String getName() {
		return name;
	}

	/**
	 * sets the assistant to either active or not
	 * @param active
	 */
	public void setActive(boolean active) {
		this.active = active;
	}
	
	/**
	 * returns true if the assistant is active, false otherwise
	 * @return
	 */
	public boolean isActive() {
		return active;
	}

	/**
	 * sets the set of skills for this assistant
	 * @param skills
	 */
	public void setSkills(HashSet<Skill> skills) {
		this.skills = skills;
	}

	/**
	 * returns the set of skills for this assistant
	 * @return
	 */
	public HashSet<Skill> getSkills() {
		return skills;
	}
	
	/**
	 * adds an unavailable time
	 * returns false if it already exists
	 * returns true if if does not exists, and adds in the new time
	 * @param time
	 * @return
	 */
	public boolean addUnavailableTime(TimeBlock time) {
		boolean alreadyExists= false;
		for (TimeBlock otherTime : unavailableTimes) {
			if (otherTime.sameTime(time))
				alreadyExists= true;
		}
		if (alreadyExists) {
			return false;
		} else {
			this.unavailableTimes.add(time);
			return true;
		}
	}

	/**
	 * sets the unavailablet times of an assistant
	 * @param unavailableTimes
	 */
	public void setUnavailableTimes(HashSet<TimeBlock> unavailableTimes) {
		this.unavailableTimes = unavailableTimes;
	}

	/**
	 * returns the unavailable times of an assistant
	 * @return
	 */
	public HashSet<TimeBlock> getUnavailableTimes() {
		return unavailableTimes;
	}

	/**
	 * sets the student limit
	 * @param studentsLimit
	 */
	public void setStudentsLimit(int studentsLimit) {
		this.studentsLimit = studentsLimit;
	}

	/**
	 * returns the student limit
	 * @return
	 */
	public int getStudentsLimit() {
		return studentsLimit;
	}

	/**
	 * adds a skill
	 * @param skill
	 */
	public void addSkill(Skill skill) {
		skills.add(skill);
	}

	/**
	 * returns true if an assistant is unavailable for a particular time, false otherwise
	 * @param time
	 * @return
	 */
	public boolean unavailableFor(TimeBlock time) {
		for (TimeBlock aTime : unavailableTimes) {
			if (aTime.intersect(time))
				return true;
		}
		return false;
	}

	/**
	 * removes a time from the list of unavailable times
	 * @param aTime
	 */
	public void removeUnavailableTime(TimeBlock aTime) {
		unavailableTimes.remove(aTime);
	}

	/**
	 * removes a skill from the list of assistant's skills
	 * @param selectedSkill
	 */
	public void removeSkill(Skill selectedSkill) {
		skills.remove(selectedSkill);
	}

	/**
	 * returns true if an assistant has the given skill, false otherwise
	 * @param selectedSkill
	 * @return
	 */
	public boolean hasSkill(Skill selectedSkill) {
		for (Skill aSkill : skills) {
			if (aSkill.getName().equals(selectedSkill.getName())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * returns true if the assistant is unavailable for a list of times, false otherwise
	 * @param time
	 * @return
	 */
	public boolean unavailableFor(ArrayList<TimeBlock> time) {
		for (TimeBlock t : time) {
			for (TimeBlock aTime : unavailableTimes) {
				if (aTime.intersect(t))
					return true;
			}
		}
		return false;
	}
	
}
