package tlass.project.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;


public class Instructor implements Serializable {

	private String name;
	private String id;
	private ArrayList<Assistant> forbiddenAssistants;
	
	/**
	 * Constructor
	 */
	public Instructor() {
		forbiddenAssistants= new ArrayList<Assistant>();
		this.name= "New Instructor";
		this.id= "eecisName";
	}
	
	/**
	 * Creates a copy of an Instructor
	 * @return
	 */
	public Instructor copy() {
		Instructor clone = new Instructor();
		clone.setForbiddenAssistants((ArrayList<Assistant>) forbiddenAssistants.clone());
		clone.setId(id);
		clone.setName(name);
		return clone;
	}
	
	/**
	 * Removes any assistants still in the instructors forbidden assistants list after that assistant
	 * has been deleted from the master list
	 * @param assistants
	 */
	public void clean(ArrayList<Assistant> assistants) {
		forbiddenAssistants.retainAll(assistants);
	}
	
	/**
	 * Restores all information to a previous backup version of an Instructor
	 * @param backup
	 */
	public void restore(Instructor backup) {
		forbiddenAssistants= backup.getForbiddenAssistants();
		id= backup.getId();
		name= backup.getName();
	}
	
	/**
	 * Sets name
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Returns name
	 * @return
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets id
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * Returns id
	 * @return
	 */
	public String getId() {
		return id;
	}
	
	/**
	 * Sets the forbidden Assistants
	 * @param forbiddenAssistants
	 */
	public void setForbiddenAssistants(ArrayList<Assistant> forbiddenAssistants) {
		this.forbiddenAssistants = forbiddenAssistants;
	}
	
	/**
	 * Returns a list of forbidden Assistants
	 * @return
	 */
	public ArrayList<Assistant> getForbiddenAssistants() {
		return forbiddenAssistants;
	}
	
	/**
	 * checks to see if an assistant is in that Instructor's forbidden assistants list
	 * @param anAssistant
	 * @return
	 */
	public boolean isForbiddenAssistant(Assistant anAssistant) {
		return forbiddenAssistants.contains(anAssistant);
	}
	
	/**
	 * Adds a forbidden assistant to the instructor's list
	 * @param assistant
	 */
	public void addForbiddenAssistant(Assistant assistant) {
		forbiddenAssistants.add(assistant);
	}
	
	/**
	 * Returns the name of the instructor when just referring to Instructor
	 */
	public String toString() {
			return name;
	}
	
	/**
	 * Changes an assistant from being on the forbidden list to now be allowed
	 * @param a
	 */
	public void allowAssistant(Assistant a) {
		Iterator<Assistant> ai = forbiddenAssistants.listIterator();
		while (ai.hasNext()) {
			Assistant anAssistant= ai.next();
			if (anAssistant == a) {
				ai.remove();
			}
		}
	}
	
}