package tlass.project.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;

import tlass.project.constraintTypes.ForbidAssignmentConstraint;
import tlass.project.constraintTypes.ForbidInstructorAssignmentConstraint;
import tlass.project.constraintTypes.LimitStudentTotalConstraint;
import tlass.project.constraintTypes.PreventScheduleConflictsConstraint;
import tlass.project.constraintTypes.PreventSectionConflictsConstraint;
import tlass.project.constraintTypes.RequireAssignmentConstraint;
import tlass.project.constraintTypes.RequireTagsConstraint;


public class Model implements Serializable{

	private static final long serialVersionUID = 3188588040290352832L;
	private ArrayList<Assistant> assistants;
	private ArrayList<Section> sections;
	private ArrayList<Schedule> schedules;
	private ArrayList<Constraint> constraints;
	private ArrayList<Instructor> instructors;
	private ArrayList<Skill> skills;
	private String semester;
	private boolean active;
	
	private int scheduleCount;
	
	/**
	 * Adds the specified skill to the model.
	 * @param aSkill
	 */
	public void addSkill(Skill aSkill) {
		skills.add(aSkill);
	}
	
	/**
	 * Adds the specified time to the model.
	 * @param atime
	 */
	public void addTime(TimeBlock atime){
		
	}
	
	/**
	 * Gets the list of skills that are currently stored in the model.
	 * @return
	 */
	public ArrayList<Skill> getSkills() {
		return skills;
	}
	
	/**
	 * Gets all of the positions currently in the model.
	 * @return
	 */
	public Position[] getPositions() {
		ArrayList<Position> positions= new ArrayList<Position>();
		for (Section aSection : sections) {
			positions.addAll(aSection.getTAs());
			positions.addAll(aSection.getLAs());
		}
		return positions.toArray(new Position[positions.size()]);
	}

	/**
	 * Gets all of the constraints currently in the model.
	 * @return
	 */
	public ArrayList<Constraint> getConstraints() {
		return constraints;
	}

	/**
	 * Sets the constraints currently in the model to the supplied list.
	 * @param constraints
	 */
	public void setConstraints(ArrayList<Constraint> constraints) {
		this.constraints = constraints;
	}

	/**
	 * Gets the list of all of the sections currently in the model.
	 * @return
	 */
	public ArrayList<Section> getSections() {
		return sections;
	}

	/**
	 * Sets the list of sections in the model to the list supplied.
	 * @param sections
	 */
	public void setSections(ArrayList<Section> sections) {
		this.sections = sections;
	}

	/**
	 * Gets the list of all schedules in the model.
	 * @return
	 */
	public ArrayList<Schedule> getSchedules() {
		return schedules;
	}

	/**
	 * Sets the list of schedules that are currently in the model to the list supplied.
	 * @param schedules
	 */
	public void setSchedules(ArrayList<Schedule> schedules) {
		this.schedules = schedules;
	}

	/**
	 * Sets the list of assistants that are currently in the model to the list supplied.
	 * @param assistants
	 */
	public void setAssistants(ArrayList<Assistant> assistants) {
		this.assistants = assistants;
	}

	/**
	 * Constructor for the MOdel.
	 */
	public Model() {
		constraints = new ArrayList<Constraint>();
		assistants = new ArrayList<Assistant>();
		sections = new ArrayList<Section>();
		schedules = new ArrayList<Schedule>();
		instructors= new ArrayList<Instructor>();
		skills= new ArrayList<Skill>();
		scheduleCount= 1;
	}
	
	/**
	 * Sets the current semester to the one specified.
	 * @param semester
	 */
	public void setSemester(String semester) {
		this.semester = semester;
	}

	/**
	 * Gets the current semester.
	 * @return
	 */
	public String getSemester() {
		return semester;
	}
	
	/**
	 * Create some filler data for simple testing purposes.
	 */
	public void createDummyData() {
		setSemester("Dummy Semester");
		
		String[] assistantNames= {"Austin Bart", "Bruce Wayne", "John Lennon", "Lelouch Lamperouge",
						"Daniel Boone", "John Cleese", "Martha Stewart", "Alan Turing",
						"Ada Lovelace"};
		String[] userNames= {"acbart", "notbatman", "sgtpepper", "lulukun",
				"appleaday", "ministerofsillywalks", "attempttolooklater", "computergod",
				"computergoddess"};
		for (int i= 0; i < assistantNames.length; i+= 1) {
			Assistant a= new Assistant();
			a.setName(assistantNames[i]);
			a.setUsername(userNames[i]);
			a.setStudentsLimit(100);
			addAssistant(a);
		}
		
		String[] instructorNames= {"Dr. Manhattan", "Dr. Dre", "Dr. Horrible"};
		for (int i= 0; i < instructorNames.length; i+= 1) {
			Instructor in= new Instructor();
			in.setName(instructorNames[i]);
			in.setId(instructorNames[i]);
			addInstructor(in);
		}
		
		String[] sectionsNames = {"CISC-475-010", "CISC-108-080", "CISC-220-080", "CISC-374-010"} ;
		int[] taCounts= {1, 1, 3, 1};
		int[] laCounts= {0, 2, 0, 1};
		int[] classCounts= {30, 30, 30, 30};
		TimeBlock[] times= {new TimeBlock(3, 0,15,5,1),
				new TimeBlock(12, 4,10,5,11),
				new TimeBlock(5, 6,4,10,5),
				new TimeBlock(7, 9,3,10,7)};
		for (int i= 0; i < sectionsNames.length; i+= 1) {
			Section s = new Section();
			s.setInstructor(instructors.get(i % instructors.size()));
			s.setId(sectionsNames[i]);
			s.setEnrolledMax(classCounts[i]);
			s.addTime(times[i]);
			for (int las= 0; las < laCounts[i]; las+= 1) {
				s.addLAPosition();
			}
			for (int tas= 0; tas < taCounts[i]; tas+= 1) {
				s.addTAPosition();
			}
			addSection(s);
		}
		sections.get(0).addRequiredAssistant(sections.get(0).getTAs().get(0),assistants.get(1));
		sections.get(1).addForbiddenAssistant(sections.get(1).getTAs().get(0),assistants.get(0));
		sections.get(1).addForbiddenAssistant(sections.get(1).getTAs().get(0),assistants.get(1));
		sections.get(1).addForbiddenAssistant(sections.get(1).getTAs().get(0),assistants.get(2));

		sections.get(0).setLocation("Smith Hall");
		sections.get(1).setLocation("Sharp Lab");
		sections.get(2).setLocation("Memorial Hall");
		sections.get(3).setLocation("Pearson Hall");

		String[] skillNames= {"Software Engineering", "English Proficiency", "Graduate Level"};
		for (String aSkillName : skillNames) {
			Skill aSkill= new Skill(aSkillName);
			addSkill(aSkill);
		}
		sections.get(0).addSkill(skills.get(0));
		sections.get(0).addSkill(skills.get(2));
		sections.get(1).addSkill(skills.get(1));
		sections.get(3).addSkill(skills.get(0));
		//sections.get(2).addSkill(new Skill("Impossible"));
		
		for(int i = 0; i<=6; i++){
			assistants.get(i).addSkill(skills.get(1));
		}		
		assistants.get(0).addSkill(skills.get(0));
		assistants.get(0).addSkill(skills.get(2));
		assistants.get(1).addSkill(skills.get(2));
		assistants.get(1).addSkill(skills.get(0));
		assistants.get(2).addSkill(skills.get(0));
		assistants.get(3).addSkill(skills.get(2));	
		assistants.get(4).addSkill(skills.get(2));
		assistants.get(6).addSkill(skills.get(0));		
		assistants.get(6).addSkill(skills.get(2));
		assistants.get(7).addSkill(skills.get(2));
		assistants.get(8).addSkill(skills.get(2));
		
		/*assistants.get(0).addUnavailableTime(new TimeBlock(2, 3));
		assistants.get(0).addUnavailableTime(new TimeBlock(5, 9));
		assistants.get(1).addUnavailableTime(new TimeBlock(7, 12));
		assistants.get(2).addUnavailableTime(new TimeBlock(12, 4));
		assistants.get(3).addUnavailableTime(new TimeBlock(7, 9));*/
		
		instructors.get(1).addForbiddenAssistant(assistants.get(2));

		constraints.add(new ForbidAssignmentConstraint());
		constraints.add(new ForbidInstructorAssignmentConstraint());
		constraints.add(new RequireAssignmentConstraint());
		constraints.add(new LimitStudentTotalConstraint());
		constraints.add(new RequireTagsConstraint());
		constraints.add(new PreventSectionConflictsConstraint());
		constraints.add(new PreventScheduleConflictsConstraint());

	}


	/**
	 * Adds the given constraint to the model
	 * @param c
	 */
	public void addConstraint(Constraint c) {
		constraints.add(c);
	}

	/**
	 * Adds the supplied schedule to the model.
	 * @param aSchedule
	 */
	public void addSchedule(Schedule aSchedule) {
		aSchedule.setName("#"+scheduleCount);
		schedules.add(aSchedule);
		scheduleCount+= 1;
	}

	/**
	 * Gets the list of assistants that are currently in the model.
	 * @return
	 */
	public ArrayList<Assistant> getAssistants() {
		return assistants;
	}

	/**
	 * Adds the supplied assistant to the list of assistants that are currently in the model.
	 * @param assistant
	 */
	public void addAssistant(Assistant assistant) {
		assistants.add(assistant);
	}

	/**
	 * Sets the list of instructors currently in the model to the list that is supplied.
	 * @param instructors
	 */
	public void setInstructors(ArrayList<Instructor> instructors) {
		this.instructors = instructors;
	}

	/**
	 * Gets the list of instructors that are currently in the model.
	 * @return
	 */
	public ArrayList<Instructor> getInstructors() {
		return instructors;
	}

	/**
	 * Gets the number of assistants that are currently in the model.
	 * @return
	 */
	public int getAssistantsCount() {
		return assistants.size();
	}

	/**
	 * Retrieves a the assistant at the specified index from the list of assistants.
	 * @param assistantIndex
	 * @return
	 */
	public Assistant getAssistant(int assistantIndex) {
		return assistants.get(assistantIndex);
	}

	/**
	 * Returns the list of assistants in the model as an array, rather than
	 * as an arraylist.
	 * @return
	 */
	public Assistant[] getAssistantsArray() {
		return assistants.toArray(new Assistant[assistants.size()]);
	}
	
	/**
	 * Returns the list of sections in the modeel as an array,
	 * rather than as an arrayList.
	 * @return
	 */
	public Section[] getSectionsArray() {
		return sections.toArray(new Section[sections.size()]);
	}

	/**
	 * Adds a new Section to the model, adding appropriate references to other components
	 * in the Model that require them.
	 * @param s
	 */
	public void addSection(Section s) {
		sections.add(s);
	}

	/**
	 * Adds a new Instructor to the model.
	 * @param i
	 */
	public void addInstructor(Instructor i) {
		instructors.add(i);
	}
	
	/**
	 * Retrieves a the schedules in the list of schedules
	 * that are marked as published. If there are none returns null
	 * @return
	 */
	public Schedule getPublishedSchedule() {
		for (Schedule s : schedules) {
			if (s.isPublished()) {
				return s;
			}
		}
		return null;
	}
	
	/**
	 * Retrieves the schedules in the list of schedules
	 * that are marked as finalized. If there are none returns nlull
	 * @return
	 */
	public Schedule getFinalizedSchedule() {
		for (Schedule s : schedules) {
			if (s.isFinalized()) {
				return s;
			}
		}
		return null;
	}

	/**
	 * Returns the skill in the model that has the same name as
	 * the name that is supplied, if it exists in the model.
	 * @param selectedSkill
	 * @return
	 */
	public Skill getSkillByName(String selectedSkill) {
		for (Skill aSkill : skills) {
			if (aSkill.getName().equals(selectedSkill))
				return aSkill;
		}
		return null;
	}
	
	/**
	 * Returns the assistant in the model that has the same name
	 * as the one that is supplied, if it exists in the model.
	 * @param s
	 * @return
	 */
	public Assistant getAssistantByName(String s){
		for(Assistant anAssistant : assistants){
			if(anAssistant.getName().equals(s));
				return anAssistant;
		}
		return null;
	}
	
	/**
	 * Returns the assistant in the model that has the same
	 * username as the one supplied, if it exists in the model.
	 * @param s
	 * @return
	 */
	public Assistant getAssistantByUsername(String s){
		for(Assistant anAssistant : assistants){
			if(anAssistant.getUsername().equals(s)) {
				return anAssistant;
			}
		}
		return null;
	}
	
	/**
	 * Returns the position in the model that has the same
	 * ID as the one supplied, if it exists in the model.
	 * @param p
	 * @return
	 */
	public Position getPositionById(String p) {
		for(Position aPosition : getPositions()){
			System.out.println(aPosition.getId()+" "+p);
			if (aPosition.getId().equals(p))
				return aPosition;
		}
		return null;
	}

	/**
	 * Not only removes the assistant from the Model, but also ensures that there
	 * are no references to the Assistant in any component of the model.
	 * @param a
	 */
	public void removeAssistant(Assistant a) {
		assistants.remove(a);
		removeAssistantFromSections(a);
		removeAssistantFromSchedules(a);
		removeAssistantFromInstructors(a);
	}

	/**
	 * Removes all references to an Assistant in all Instructors
	 * @param a
	 */
	private void removeAssistantFromInstructors(Assistant a) {
		for (Instructor anInstructor : instructors) {
			anInstructor.allowAssistant(a);
		}
	}

	/**
	 * Removes all references to an Assistant in all Sections
	 * @param a
	 */
	private void removeAssistantFromSections(Assistant a) {
		for (Section aSection : sections) {
			aSection.allowAssistant(a);
			aSection.optionalizeAssistant(a);
		}
	}

	/**
	 * Removes all references to an Assistant in all Schedules
	 * @param a
	 */
	private void removeAssistantFromSchedules(Assistant a) {
		for (Schedule aSchedule : schedules) {
			aSchedule.unassignAssistant(a);
		}
	}
	/**
	 * Renames an assistant to a specified name.
	 * @param a
	 * @param name
	 */
	public void renameAssistant(Assistant a, String name){
		int index = assistants.indexOf(a);
		Assistant placeHold = assistants.get(index);
		placeHold.setName(name);
		assistants.set(index, placeHold);
		
	}
	/**
	 * Removes an instructor from the Model, also ensuring that there
	 * are no references to the instructor in any component of the Model.
	 * @param i
	 */
	public void removeInstructor(Instructor i) {
		instructors.remove(i);
		removeInstructorFromSections(i);
	}
	
	/**
	 * Rename the instructor to a given name.
	 * @param i
	 * @param name
	 */
	public void renameInstructor(Instructor i, String name){
		int index = instructors.indexOf(i);
		Instructor placeHold = instructors.get(index);
		placeHold.setName(name);
		instructors.set(index, placeHold);
	}
	
	/**
	 * Removes all references to an instructor from all sections
	 * @param i
	 */
	private void removeInstructorFromSections(Instructor i) {
		for (Section aSection : sections) {
			if (aSection.getInstructor() == i) {
				aSection.setInstructor(null);
			}
		}
	}

	/**
	 * Removes a section from the Model, also ensuring that there
	 * are no references to the section in any component of the Model.
	 * @param s
	 */
	public void removeSection(Section s) {
		sections.remove(s);
		removeSectionFromSchedules(s);
	}
	
	/**
	 * Renames a Section to a specified name.
	 * @param s
	 * @param name
	 */
	public void renameSection(Section s, String name){
		int index = sections.indexOf(s);
		Section placeHold = sections.get(index);
		placeHold.setId(name);
		sections.set(index, placeHold);
		
	}
	
	/**
	 * Removes a section from all the Schedules in the model.
	 * @param s
	 */
	private void removeSectionFromSchedules(Section s) {
		for (Schedule aSchedule : schedules) {
			aSchedule.unassignSection(s);
		}
	}

	/**
	 * Removes a Schedule from the Model, ensuring that there are no references to the
	 * Schedule elsewhere in the model.
	 * @param s
	 */
	public void removeSchedule(Schedule s) {
		schedules.remove(s);
	}
	
	/**
	 * Renames the schedule to a specified name.
	 * @param s
	 * @param name
	 */
	public void renameSchedule(Schedule s, String name){
		int index = schedules.indexOf(s);
		Schedule placeHold = schedules.get(index);
		placeHold.setName(name);
		schedules.set(index, placeHold);
	}

	/**
	 * Looking at the ordered list of sections and their consistent ordering of Positions,
	 * returns a specific Position.
	 * @param row
	 * @return a specific Position
	 */
	public Position getPositionAt(int row) {
		return getPositions()[row];
	}

	/**
	 * Replaces all the data in the model with the data from the given model
	 * @param newModel
	 */
	public void loadModel(Model newModel) {
		assistants= newModel.getAssistants();
		sections= newModel.getSections();
		schedules= newModel.getSchedules();
		constraints= newModel.getConstraints();
		instructors= newModel.getInstructors();
		skills= newModel.getSkills();
		semester= newModel.getSemester();
		scheduleCount= newModel.getScheduleCount();
		active= newModel.isActive();
	}

	/**
	 * Gets the number of schedules that currently exsist in the model.
	 * @return
	 */
	private int getScheduleCount() {
		return scheduleCount;
	}

	/**
	 * Returns the sections that are currently in the model
	 * they are sorted alphabetically.
	 * @return
	 */
	public ArrayList<Section> getSortedSections() {
		ArrayList<Section> sortedSectionList= (ArrayList<Section>) sections.clone(); 
		Collections.sort(sortedSectionList, new Comparator<Section>() {
			@Override
			public int compare(Section s1, Section s2) {
				return s1.toString().compareTo(s2.toString());
			}
		});
		return sortedSectionList;
	}

	/**
	 * Queries the model to find if a skill of the same name
	 * as the one supplied exists.
	 * @param newSkill
	 * @return
	 */
	public boolean skillExists(Skill newSkill) {
		for (Skill aSkill : skills) {
			if (aSkill.getName().equals(newSkill.getName())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Queries the model to see if the skill of the supplied name
	 * is referenced.
	 * @param selectedSkill
	 * @return
	 */
	public boolean skillReferenced(Skill selectedSkill) {
		boolean atLeastOneExists= false;
		for (Assistant a : assistants) {
			if (a.hasSkill(selectedSkill))
				atLeastOneExists= true;
		}
		for (Section s : sections) {
			if (s.hasSkill(selectedSkill))
				atLeastOneExists= true;
		}
		return atLeastOneExists;
	}

	/**
	 * Removes the skill of the name matching the supplied name 
	 * from the model.
	 * @param selectedSkill
	 */
	public void removeSkill(Skill selectedSkill) {
		skills.remove(selectedSkill);
	}

	/**
	 * Sets the activity state of the model.
	 * @param active
	 */
	public void setActive(boolean active) {
		this.active = active;
	}

	/**
	 * Retrieves the activity state of the model.
	 * @return
	 */
	public boolean isActive() {
		return active;
	}

	/**
	 * Removes the specified position from the list of available
	 * ta positions from the model.
	 * @param position
	 */
	public void removeTAPosition(Position position) {
		Section section= position.getSection();
		section.removeTA(position);
	}
	
	/**
	 * Removes the specified position from the list of available
	 * la positions from the model.
	 * @param position
	 */
	public void removeLAPosition(Position position) {
		Section section= position.getSection();
		section.removeLA(position);
	}

}
