package tlass.project.model;

import java.io.Serializable;

import tlass.project.gui.panels.Restriction;

public class Position implements Serializable {

	private Section section;
	private boolean isTA;
	private int order;
	
	/**
	 * Constructor for Position
	 * @param s
	 * @param isTA
	 * @param o
	 */
	public Position(Section s, boolean isTA, int o) {
		this.section= s;
		this.isTA= isTA;
		this.order= o;
	}

	/**
	 * Sets the postion to either be a TA or LA position
	 * @param isTA
	 */
	public void setTA(boolean isTA) {
		this.isTA = isTA;
	}

	/**
	 * returns true if this is a TA position, false if it is an LA position
	 * @return
	 */
	public boolean isTA() {
		return isTA;
	}

	/**
	 * Sets the section
	 * @param section
	 */
	public void setSection(Section section) {
		this.section = section;
	}

	/**
	 * returns the section
	 * @return
	 */
	public Section getSection() {
		return section;
	}

	/**
	 * Returns the name of the position
	 * @return
	 */
	public String getPositionName() {
		int position= 1+order;
		if (isTA) {
			return "TA #"+position;	
		} else {
			return "LA #"+position;
		}
	}

	/**
	 * Returns the namem of the section
	 * @return
	 */
	public String getSectionName() {
		return section.toString();
	}
	
	/**
	 * Returns the section id
	 * @return
	 */
	public String getSectionId() {
		return section.getId();
	}
	
	/**
	 * Returns 1 if this is a TA position, 0 if this is an LA position
	 * @return
	 */
	public int getType() {
		if (isTA) {
			return 1;
		} else {
			return 0;
		}
	}

	/**
	 * returns the description for this position
	 * @return
	 */
	public String getDescription() {
		return section.describe();
	}

	/**
	 * prints this position
	 */
	public String toString() {
		return "#"+(1+order);
	}
	
	/**
	 * returns the order of this position
	 * @return
	 */
	public int getOrder() {
		return order;
	}
	
	/**
	 * sets the order of this position
	 * @param i
	 */
	public void setOrder(int i) {
		order= i;
	}
	
	/**
	 * returns the id of this position
	 * @return
	 */
	public String getId() {
		if (isTA) {
			return section.getId() + "-ta-"+(1+order);
		} else {
			return section.getId() + "-la-"+(1+order);
		}
	}
	
	/**
	 * Builds the total id representation based on the sectionId, position type, and place 
	 * @param sectionId
	 * @param isTA
	 * @param place
	 * @return
	 */
	public static String buildId(String sectionId, int isTA, int place) {
		if (isTA == 1) {
			return sectionId+"-ta-"+place;
		} else {
			return sectionId+"-la-"+place;
		}
	}

	/**
	 * returns the forbidden assistants and required assistants for this section if applicable
	 * @return
	 */
	public Restriction getRestriction() {
		if (section.hasForbiddenAssistants(this)) {
			return Restriction.MULTIPLE; 
		} else if (section.hasRequiredAssistant(this)) {
			return Restriction.SINGLE;
		} else {
			return Restriction.NONE;
		}
	}

	/**
	 * returns the actual object of the forbidden and required assistants
	 * @return
	 */
	public Object getRestrictionValue() {
		if (section.hasForbiddenAssistants(this)) {
			return section.getForbiddenAssistants(this); 
		} else if (section.hasRequiredAssistant(this)) {
			return section.getRequiredAssistant(this);
		} else {
			return null;
		}
	}

}