package tlass.project.model;

import java.io.Serializable;

public class Response implements Serializable {

	public final static int UNSEEN= 0;
	public final static int ACCEPTED= 1;
	public final static int REJECTED= 2;
	public final static int OVERRIDDEN= 3;
	public final static int FINALIZED= 4;
	
	public final static String[] RESPONSES= {"Unseen", "Accepted", "Rejected", "Overridden", "Finalized"};

	private int value;
	private String comment;
	
	/**
	 * Creates an empty Response 
	 */
	public Response() {
		this.reset();
	}
	
	/**
	 * Creates a Response with given acceptance and comment values
	 * @param acceptance
	 * @param comment
	 */
	public Response(boolean acceptance, String comment) {
		if (acceptance) {
			this.value= ACCEPTED;
		} else {
			this.value= REJECTED;
			this.comment= comment;
		}
	}
	
	/**
	 * Prints the Response
	 */
	public String toString() {
		return RESPONSES[value];
	}
	
	/**
	 * returns the value of the response (5 different types)
	 * @return
	 */
	public int getValue() {
		return value;
	}
	
	/**
	 * sets the value of the response (5 different types)
	 * @return
	 */
	public void setValue(int v) {
		this.value = v;
	}
	
	/**
	 * set the value of this response to be accepted
	 */
	public void accept() {
		this.value= ACCEPTED;
	}
	
	/**
	 * set the value of this response to be rejected, and sets the comment explaining why
	 * @param comment
	 */
	public void reject(String comment) {
		this.value= REJECTED;
		this.comment= comment;
	}
	
	/**
	 * set the value of this response to be overridden
	 */
	public void override() {
		this.value= OVERRIDDEN;
	}
	
	/**
	 * set the value of this response to be finalized
	 */
	public void finalize() {
		this.value= FINALIZED;
	}
	
	/**
	 * set the value of this response to be unseen
	 */
	public void unseen() {
		this.value= UNSEEN;
	}
	
	/**
	 * sets the comment fot this response
	 * @param comment
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}
	
	/**
	 * returns the comment for this response
	 * @return
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * resents the response to Unseen and to a blank comment 
	 */
	public void reset() {
		this.value= UNSEEN;
		this.comment= "";
	}

	/**
	 * returns true if the response is accepted
	 * @return
	 */
	public boolean isAccepted() {
		return value == ACCEPTED;
	}
	/**
	 * returns true if the response is unseen
	 * @return
	 */
	public boolean isUnseen() {
		return value == UNSEEN;
	}
	
	/**
	 * returns true if the response is overriden
	 * @return
	 */
	public boolean isOverridden() {
		return value == OVERRIDDEN;
	}
	
	/**
	 * returns true if the response is rejected
	 * @return
	 */
	public boolean isRejected() {
		return value == REJECTED;
	}
	
	/**
	 * returns true if the response is finalized
	 * @return
	 */
	public boolean isFinalized() {
		return value == FINALIZED;
	}

	/**
	 * sets the response based on another response's value and comment
	 * @param response
	 */
	public void setFrom(Response response) {
		value= response.getValue();
		comment= response.getComment();
	}

	/**
	 * returns true if two responses are the same type i.e. have the same value
	 * @param response
	 * @return
	 */
	public boolean sameAs(Response response) {
		return value == response.getValue();
	}
}
