/* Section.java
 * CISC475 Project
 * Group 02
 */

package tlass.project.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map.Entry;


public class Section  implements Serializable {
	
	//variables for new system design
	private String id;
	private boolean active;
	private String location;
	private String title;
	private int enrolled;
	private int enrolledMax;
	private ArrayList<TimeBlock> time;
	private HashSet<Skill> skillsNeeded;
	private ArrayList<Position> tas;
	private ArrayList<Position> las;
	private boolean assistantsPresent;
	private Instructor instructor;
	private HashMap<Position,Assistant> requiredAssistants;
	private HashMap<Position,ArrayList<Assistant>> forbiddenAssistants;
	
	/**
	 * Returns the id.
	 */
	public String toString() {
		return id;
	}
	
	/**
	 * Constructor
	 */
	public Section() {
		this.id= "New Section";
		this.title= id;
		this.active= true;
		this.time= new ArrayList<TimeBlock>();
		this.assistantsPresent= true;
		this.tas= new ArrayList<Position>();
		this.las= new ArrayList<Position>();
		this.requiredAssistants= new HashMap<Position, Assistant>();
		this.forbiddenAssistants= new HashMap<Position, ArrayList<Assistant>>();
		this.skillsNeeded= new HashSet<Skill>();
	}
	
	/**
	 * Adds the assistant to the provided position.
	 * @param position
	 * @param a
	 */
	public void addRequiredAssistant(Position position, Assistant a) {
		this.requiredAssistants.put(position, a);
	}

	/**
	 * Sets the section to an active state.
	 * @param active
	 */
	public void setActive(boolean active) {
		this.active = active;
	}

	/**
	 * Gets whether the section is active.
	 * @return
	 */
	public boolean isActive() {
		return active;
	}

	/**
	 * Sets the location of the section.
	 * @param location
	 */
	public void setLocation(String location) {
		this.location = location;
	}

	/**
	 * Gets the location of the section.
	 * @return
	 */
	public String getLocation() {
		return location;
	}

	/**
	 * Sets the number of enrolled students for the section.
	 * @param enrolled
	 */
	public void setEnrolled(int enrolled) {
		this.enrolled = enrolled;
	}

	/**
	 * Gets the number of enrolled students for the section.
	 * @return
	 */
	public int getEnrolled() {
		return enrolled;
	}

	/**
	 * Sets the maximum number of students that can be enrolled in the section.
	 * @param enrolledMax
	 */
	public void setEnrolledMax(int enrolledMax) {
		this.enrolledMax = enrolledMax;
	}

	/**
	 * Gets the maximum number of students that can be enrolled in the section.
	 * @return
	 */
	public int getEnrolledMax() {
		return enrolledMax;
	}

	/**
	 * Sets the time for the section.
	 * @param time
	 */
	public void setTime(ArrayList<TimeBlock> time) {
		this.time = time;
	}
	
	/**
	 * Adds a time that the section
	 * @param time
	 */
	public void addTime(TimeBlock time) {
		this.time.add(time);
	}
	
	/**
	 * Removes a time that the section meets.
	 * @param time
	 */
	public void removeTime(TimeBlock time) {
		this.time.remove(time);
	}
	
	/**
	 * Checks to see if the section meets at the supplied time.
	 * @param time
	 * @return
	 */
	public boolean hasTime(TimeBlock time) {
		return this.time.contains(time);
	}
	
	/**
	 * Consumes a start hour and start minuite.
	 * Sets a time for the section to begin to meet. 
	 * @param sm
	 * @param sh
	 */
	public void setStartTime(int sm, int sh) {
		for (TimeBlock aTime : time) {
			aTime.setStartTime(sm, sh);
		}
	}
	
	/**
	 * Consumes a start hour and start minuite.
	 * Sets a time for the section to endits meeting session
	 * Sets a time 
	 * @param em
	 * @param eh
	 */
	public void setEndTime(int em, int eh) {
		for (TimeBlock aTime : time) {
			aTime.setEndTime(em, eh);
		}
	}
	
	/**
	 * Conusmes a day, start time and end time.
	 * Adds a day. A start time and end time to the section.
	 * @param day day of the week
	 * @param start start time
	 * @param end end time
	 */
	public void addDayForce(int day, int start, int end) {
		boolean added= false;
		for (TimeBlock aTime : time) {
			if (aTime.getStartDay() == day) {
				added= true;
				aTime.setStartTime(start);
				aTime.setEndTime(end);
			}
		}
		if (!added) {
			time.add(new TimeBlock(day, start, end));
		}
	}
	
	/**
	 * Consumes a day.
	 * Removes specified day from the section.
	 * @param day
	 */
	public void removeDay(int day) {
		for (TimeBlock aTime : time) {
			if (aTime.getStartDay() == day) {
				time.remove(aTime);
				return;
			}
		}
	}

	/**
	 * Gets the time for the section.
	 * @return
	 */
	public ArrayList<TimeBlock> getTime() {
		return time;
	}

	/**
	 * Sets the list of skills needed to hold a ta or la position for the section.
	 * @param skillsNeeded
	 */
	public void setSkillsNeeded(HashSet<Skill> skillsNeeded) {
		this.skillsNeeded = skillsNeeded;
	}

	/**
	 * Gets the list of skills needed to hold a ta or la position for the section.
	 * @return
	 */
	public HashSet<Skill> getSkillsNeeded() {
		return skillsNeeded;
	}

	/**
	 * Sets the TAs for the section.
	 * @param tas
	 */
	public void setTAs(ArrayList<Position> tas) {
		this.tas = tas;
	}

	/**
	 * Gets the TAs for the section.
	 * @return
	 */
	public ArrayList<Position> getTAs() {
		return tas;
	}

	/**
	 * Sets the LAs for the section.
	 * @param las
	 */
	public void setLAs(ArrayList<Position> las) {
		this.las = las;
	}

	/**
	 * Gets the LAs for the section.
	 * @return
	 */
	public ArrayList<Position> getLAs() {
		return las;
	}

	/**
	 * Set the asssistantPresent flag which tells whether the assistant is currently
	 * present for the section.
	 * @param assistantsPresent
	 */
	public void setAssistantsPresent(boolean assistantsPresent) {
		this.assistantsPresent = assistantsPresent;
	}

	/**
	 * Gets the state of the assistantsPresent flag.
	 * @return
	 */
	public boolean isAssistantsPresent() {
		return assistantsPresent;
	}

	/**
	 * Sets the Instructor for the section.
	 * @param instructor
	 */
	public void setInstructor(Instructor instructor) {
		this.instructor = instructor;
	}

	/**
	 * Gets the Instructor for the section.
	 * @return
	 */
	public Instructor getInstructor() {
		return instructor;
	}

	/**
	 * Sets a list of assistants, which are mapped to positions, who are required for the section.
	 * @param requiredAssistants
	 */
	public void setRequiredAssistants(HashMap<Position, Assistant> requiredAssistants) {
		this.requiredAssistants = requiredAssistants;
	}

	/**
	 * Gets the hashmap that contains the required assistants, mapped to positions, for the section.
	 * @return
	 */
	public HashMap<Position, Assistant> getRequiredAssistants() {
		return requiredAssistants;
	}

	/**
	 * Set the assistants which are forbidden from being mapped to specified positions;
	 * @param forbiddenAssistants
	 */
	public void setForbiddenAssistants(HashMap<Position,ArrayList<Assistant>> forbiddenAssistants) {
		this.forbiddenAssistants = forbiddenAssistants;
	}

	/**
	 * Gets the assistants which are forbidden from being mapped to specified positions.
	 * @return
	 */
	public HashMap<Position,ArrayList<Assistant>> getForbiddenAssistants() {
		return forbiddenAssistants;
	}

	/**
	 * Add a new LA position that can potentially be filled by an assistant
	 * @return
	 */
	public Position addLAPosition() {
		Position position= new Position(this, false, las.size());
		las.add(position);
		forbiddenAssistants.put(position, new ArrayList<Assistant>());
		requiredAssistants.put(position, null);
		return position;
	}
	
	/**
	 * Create a TA position which can potentially be filled by an assistant
	 * @return
	 */
	public Position addTAPosition() {
		Position position= new Position(this, true, tas.size());
		tas.add(position);
		forbiddenAssistants.put(position, new ArrayList<Assistant>());
		requiredAssistants.put(position, null);
		return position;
	}

	/**
	 * Set the id(name) of the section.
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;
		this.title= id;
	}

	/**
	 * Get the id(name) of the section.
	 * @return
	 */
	public String getId() {
		return id;
	}

	/**
	 * Checks to see if the assistant is forbidden from being paired with the specified section.
	 * @param aPosition
	 * @param anAssistant
	 * @return
	 */
	public boolean isForbiddenAssistant(Position aPosition,	Assistant anAssistant) {
		if (forbiddenAssistants.containsKey(aPosition)) {
			ArrayList<Assistant> as= forbiddenAssistants.get(aPosition);
			return as.contains(anAssistant);
		}
		return false;
	}

	/**
	 * Checks to see if the assistant and position pairing is required.
	 * @param aPosition
	 * @param anAssistant
	 * @return
	 */
	public boolean isRequiredAssistant(Position aPosition, Assistant anAssistant) {
		if (requiredAssistants.containsKey(aPosition)) {
			Assistant a= requiredAssistants.get(aPosition);
			return (a == anAssistant);
		}
		return false;
	}

	/**
	 * Pairs the specified position and assistant and marks them as a forbidden pairing.
	 * @param position
	 * @param assistant
	 */
	public void addForbiddenAssistant(Position position, Assistant assistant) {
		ArrayList<Assistant> as= forbiddenAssistants.get(position);
		as.add(assistant);
	}

	/**
	 * Add a skill that is required to be a TA or LA for the section.
	 * @param skill
	 */
	public void addSkill(Skill skill) {
		this.skillsNeeded.add(skill);
	}

	/**
	 * Checks to see if there is a time conflict
	 * @param secondSection
	 * @return
	 */
	public boolean intersect(Section secondSection) {
		for (TimeBlock myTime : time) {
			for (TimeBlock theirTime: secondSection.getTime()) {
				if (myTime.intersect(theirTime)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Returns a description of the section in plaintext string.
	 * @return
	 */
	public String describe() {
		return title+" at "+time+" in "+location+" with "+instructor;
	}

	/**
	 * Gets an ordered list of the TAs
	 * @param position
	 * @return
	 */
	public int getTAOrder(Position position) {
		return getTAs().indexOf(position);
	}
	
	/**
	 * Gets an ordered list of the LAs
	 * @param position
	 * @return
	 */
	public int getLAOrder(Position position) {
		return getLAs().indexOf(position);
	}

	/**
	 * Returns a list of the positions, both open and filled.
	 * @return
	 */
	public ArrayList<Position> getPositions() {
		ArrayList<Position> totalList = (ArrayList<Position>) las.clone();
		totalList.addAll(tas);
		return totalList;
	}

	/**
	 * Allows an assistant to be considered for a position.
	 * @param a
	 */
	public void allowAssistant(Assistant a) {
		for (Position anOp : forbiddenAssistants.keySet()) {
			Iterator<Assistant> ai = forbiddenAssistants.get(anOp).listIterator();
			while (ai.hasNext()) {
				Assistant anAssistant= ai.next();
				if (anAssistant == a) {
					ai.remove();
				}
			}
		}
	}

	/**
	 * TODO
	 * @param a
	 */
	public void optionalizeAssistant(Assistant a) {
		for (Position anOp : requiredAssistants.keySet()) {
			if (requiredAssistants.get(anOp) == a) {
				requiredAssistants.put(anOp, null);
			}
		}
	}

	/**
	 * Returns an ordered list of the Positions.
	 * @return
	 */
	public ArrayList<Position> getSortedPositions() {
		ArrayList<Position> sortedPositionList = getPositions();
		Collections.sort(sortedPositionList, new Comparator<Position>() {
			@Override
			public int compare(Position p1, Position p2) {
				if (p1.isTA() == p2.isTA()) {
					return p1.getOrder() - p2.getOrder();
				} else if (p1.isTA()) {
					return -1;
				} else {
					return 1;
				}
			}
		});
		return sortedPositionList;
	}


	/**
	 * Check to see if the supplied skill is already in the
	 * list of skills required for an assistant to be assigned
	 * to a section.
	 * @param selectedSkill
	 * @return
	 */
	public boolean hasSkill(Skill selectedSkill) {
		for (Skill aSkill : skillsNeeded) {
			if (aSkill.getName().equals(selectedSkill.getName())) {
				return true;
			}
		}
		return false;
	}


	/**
	 * Removes the supplied skill from the list of skills required
	 * for an assistant to be assigned to the section
	 * @param selectedSkill
	 */
	public void removeSkill(Skill selectedSkill) {
		skillsNeeded.remove(selectedSkill);
	}


	/**
	 * Checks to see if the supplied position has any assistants
	 * forbidden from being assigned to it.
	 * @param position
	 * @return
	 */
	public boolean hasForbiddenAssistants(Position position) {
		return !forbiddenAssistants.get(position).isEmpty();
	}
	
	/**
	 * Checks to see if the supplied position has any assistants
	 * that are required to be assingned to it.
	 * @param position
	 * @return
	 */
	public boolean hasRequiredAssistant(Position position) {
		return requiredAssistants.get(position) != null;
	}

	/**
	 * Returns an ArrayList of the assistants forbidden
	 * from being assigned to the supplied position
	 * @param position
	 * @return
	 */
	public ArrayList<Assistant> getForbiddenAssistants(Position position) {
		return forbiddenAssistants.get(position);
	}
	
	/**
	 * Reutrns the assistant that is required to be assigned
	 * to the supplied position
	 * @param position
	 * @return
	 */
	public Assistant getRequiredAssistant(Position position) {
		return requiredAssistants.get(position);
	}

	/**
	 * Consumes an ArrayLIst of assistants, skills, and instructors
	 * and then removes all of the elements from the three
	 * arraylists
	 * @param assistants
	 * @param skills
	 * @param instructors
	 */
	public void clean(ArrayList<Assistant> assistants, ArrayList<Skill> skills, ArrayList<Instructor> instructors) {
		for (Skill aSkill: skillsNeeded) {
			if (!skills.contains(aSkill)) {
				skillsNeeded.remove(aSkill);
			}
		}
		if (instructors.contains(instructor)) {
			instructor= null;
		}
		for (Entry<Position, Assistant> e : requiredAssistants.entrySet()) {
			if (assistants.contains(e.getValue())) {
				requiredAssistants.put(e.getKey(), null);
			}
		}
		for (Entry<Position, ArrayList<Assistant>> e : forbiddenAssistants.entrySet()) {
			ArrayList<Assistant> as= forbiddenAssistants.get(e.getKey());
			if (as.contains(e.getValue())) {
				as.remove(e.getValue());
			}
		}		
	}

	/**
	 * Restores the section data to the data stored in the backup,
	 * reverting any changes
	 * @param sectionBackup
	 */
	public void restore(Section sectionBackup) {
		id= sectionBackup.getId();
		active= sectionBackup.isActive();
		location= sectionBackup.getLocation();
		title= sectionBackup.getTitle();
		enrolled= sectionBackup.getEnrolled();
		enrolledMax= sectionBackup.getEnrolledMax();
		time= sectionBackup.getTime();
		skillsNeeded= sectionBackup.getSkillsNeeded();
		tas= sectionBackup.getTAs();
		las= sectionBackup.getLAs();
		assistantsPresent= sectionBackup.getAssistantsPresent();
		instructor= sectionBackup.getInstructor();
		requiredAssistants= sectionBackup.getRequiredAssistants();
		forbiddenAssistants= sectionBackup.getForbiddenAssistants();
	}
	
	/**
	 * Returns true if there are assistants presently
	 * assigned to the section
	 * @return
	 */
	private boolean getAssistantsPresent() {
		return assistantsPresent;
	}

	/**
	 * Returns the title of section.
	 * @return
	 */
	private String getTitle() {
		return title;
	}

	/**
	 * Clones all of the data for the section
	 * @return
	 */
	public Section copy() {
		Section clone = new Section();
		clone.setId(id);
		clone.setActive(active);
		clone.setLocation(location);
		clone.setTitle(title);
		clone.setEnrolled(enrolled);
		clone.setEnrolledMax(enrolledMax);
		clone.setTime((ArrayList<TimeBlock>) time.clone());
		clone.setSkillsNeeded((HashSet<Skill>) skillsNeeded.clone());
		clone.setTAs((ArrayList<Position>) tas.clone());
		clone.setLAs((ArrayList<Position>) las.clone());
		clone.setAssistantsPresent(assistantsPresent);
		clone.setInstructor(instructor);
		clone.setRequiredAssistants((HashMap<Position, Assistant>) requiredAssistants.clone());
		clone.setForbiddenAssistants((HashMap<Position, ArrayList<Assistant>>) forbiddenAssistants.clone());
		return clone;
	}

	/**
	 * Sets the title of the section to the one specified.
	 * @param title2
	 */
	private void setTitle(String title2) {
		title= title2;
	}

	/**
	 * Reomoves the selected Ta Position from the section.
	 * @param position
	 */
	public void removeTA(Position position) {
		int newStart= position.getOrder();
		tas.remove(position);
		for (int i= newStart; i < tas.size(); i+= 1) {
			tas.get(i).setOrder(i);
		}
	}
	
	/**
	 * Removes the selected La position from the section.
	 * @param position
	 */
	public void removeLA(Position position) {
		int newStart= position.getOrder();
		las.remove(position);
		for (int i= newStart; i < las.size(); i+= 1) {
			las.get(i).setOrder(i);
		}
	}

	/**
	 * Returns the time that the section meets.
	 * @return
	 */
	public TimeBlock getCurrentTime() {
		if (time.isEmpty()) {
			return new TimeBlock(0, 0, 0);
		} else {
			return time.get(0);
		}
	}

	/**
	 * Checks to see if section meets on the supplied day.
	 * @param day
	 * @return
	 */
	public boolean hasDay(int day) {
		for (TimeBlock aTime : time) {
			if (aTime.getStartDay() == day) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Sets the time that the section meets.
	 * @param start
	 * @param end
	 */
	public void setTime(int start, int end) {
		for (TimeBlock aTime : time) {
			aTime.setStartTime(start);
			aTime.setEndTime(end);
		}
	}

	/**
	 * Sets the assistant as required for the selected position for this section.
	 * @param position
	 * @param data
	 */
	public void setRequiredAssistants(Position position, Assistant data) {
		requiredAssistants.put(position, data);
	}

	/**
	 * Sets the assistant as forbidden for the selected position for the section.
	 * @param position
	 * @param data
	 */
	public void setForbiddenAssistants(Position position,
			ArrayList<Assistant> data) {
		ArrayList<Assistant> fas= forbiddenAssistants.get(position);
		fas.clear();
		fas.addAll(data);
	}
	
}
