package tlass.unitTesting.gui;

import static org.junit.Assert.*;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import tlass.project.Controller;
import tlass.project.gui.LoginWindow;

/**
 * 
 * @author Michael Chinn
 *
 */
public class TestLoginWindow {
	private static Controller c;
	private static JFrame jf;
	private static String s;
	private LoginWindow lw;
	
	/**
	 * create a string, controller and jframe for use in the class
	 * 
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		c = new Controller();
		jf = new JFrame();
		s = new String("test");
	}

	/**
	 * null the string, controller and jframe after the class is over
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		c = null;
		jf = null;
		s = null;
	}

	/**
	 * create a login window instance for each test
	 * 
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		lw = new LoginWindow(jf,c);
	}

	/**
	 * null the login window after each test
	 * 
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		lw = null;
	}

	/**
	 * make sure the constructor makes the login window have 3 components
	 */
	@Test
	public void componentCountTest() {
		assertEquals("Login Window should have 3 components",3,lw.getComponentCount());
	}
	
	/**
	 * make sure that we can get hte text field and inteeract with it successfully
	 */
	@Test
	public void getLoginButtonTest() {
		assertEquals("Login button should be JButton",JButton.class,lw.getLoginButton().getClass());
		assertEquals("Login button text should be Login","Login",lw.getLoginButton().getText());
		assertNotSame("Login button text should be Login","",lw.getLoginButton().getText());
	}

	/**
	 * make sure that we can get the text field successfully and interact with it
	 */
	@Test
	public void getUserTest() {
		assertEquals("User text field should be JTextField",JTextField.class,lw.getUser().getClass());
		assertEquals("User text field text should be ''","",lw.getUser().getText());
		assertNotSame("User text field text should be ''","test",lw.getUser().getText());
		lw.getUser().setText(s);
		assertEquals("User text field text should be 'test'",s,lw.getUser().getText());
		assertNotSame("User text field text should not be the same as s","test",lw.getUser().getText());
	}
	
	/**
	 * make sure that we can get the password field successfully and interact with it
	 */
	@Test
	public void getPassTest() {
		assertEquals("Password field should be JPasswordField",JPasswordField.class,lw.getPass().getClass());
		assertEquals("Password field text should be ''","",new String(lw.getPass().getPassword()));
		assertNotSame("Password field text should be ''","test",new String(lw.getPass().getPassword()));
		lw.getPass().setText(s);
		assertEquals("User text field text should be 'test'",s,new String(lw.getPass().getPassword()));
		assertNotSame("User text field text should not be the same as s","test",new String(lw.getPass().getPassword()));
	}
}