package tlass.unitTesting.gui.panels;

import static org.junit.Assert.*;

import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import tlass.project.Controller;
import tlass.project.gui.MainWindow;
import tlass.project.gui.panels.AssistantPanel;
import tlass.project.model.Assistant;

public class TestAssistantPanel {
	private AssistantPanel ap;
	private static Controller c;
	private static Assistant a;
	private static JTree st;
	private static DefaultMutableTreeNode n;
	private static MainWindow mw;

	/**
	 * setup controller, assistant, jtree, default mutabletreenode and main window for the class
	 * 
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		c = new Controller();
		a = new Assistant();
		st = new JTree();
		n = new DefaultMutableTreeNode();
		mw = new MainWindow(c);
	}

	/**
	 * null controller, assistant, jtree, default mutabletreenode and main window after the class
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		c = null;
		a = null;
		st = null;
		n = null;
		mw = null;
	}

	/**
	 * create an assistant panel for each test
	 * 
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		ap = new AssistantPanel(c,a,st,n,mw);
	}

	/**
	 * null the assistant panel after each test
	 * 
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		ap = null;
	}

	/**
	 * test how many components are put in an assistant panel when constructed
	 */
	@Test
	public void getAssistantTest() {
		assertEquals("Assistant a should be in the assistant panel",a,ap.getAssistant());
	}

	/**
	 * test how many components are put in an assistant panel when constructed
	 */
	@Test
	public void setAssistantTest() {
		Assistant newA = new Assistant();
		assertSame("There should be 2 components",a,ap.getAssistant());
		assertNotSame("There should be 2 components",newA,ap.getAssistant());
		ap.setAssistant(newA);
		assertNotSame("There should be 2 components",a,ap.getAssistant());
		assertSame("There should be 2 components",newA,ap.getAssistant());
	}
	
	/**
	 * test how many components are put in an assistant panel when constructed
	 */
	@Test
	public void getToolBarTest() {
		assertEquals("There should be 2 components",2,ap.getToolBar().getComponentCount());
	}
	
	/**
	 * test how many components are put in an assistant panel when constructed
	 */
	@Test
	public void setToolBarTest() {
		JToolBar jtb = new JToolBar();
		ap.setToolBar(jtb);
		assertNotSame("There should be 2 components",2,ap.getToolBar().getComponentCount());
		assertEquals("There should be 0 components",0,ap.getToolBar().getComponentCount());
	}
}