package tlass.unitTesting.gui.panels;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import tlass.project.Controller;
import tlass.project.constraintTypes.ForbidAssignmentConstraint;
import tlass.project.constraintTypes.ForbidInstructorAssignmentConstraint;
import tlass.project.gui.panels.ConstraintPanel;
import tlass.project.model.Constraint;

/**
 * 
 * @author Michael Chinn
 *
 */
public class TestConstraintPanel {
	private static Controller controller;
	private static Constraint constraint;
	private ConstraintPanel cp;

	/**
	 * create a controller and constraint for use in the class
	 * 
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		controller = new Controller();
		constraint = new ForbidAssignmentConstraint();
		
	}

	/**
	 * null the controller and constraint after done with the class
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		controller = null;
		constraint = null;
	}

	/**
	 * create a constraint panel for each test
	 * 
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		cp = new ConstraintPanel(controller,constraint);
	}

	/**
	 * null the constraint panel after each test
	 * 
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		cp = null;
	}

	/**
	 * check and make sure there is 1 component in the panel
	 */
	@Test
	public void componentCountTest() {
		assertEquals("ConstraintPanel should have 1 component",1,cp.getComponentCount());
	}
	
	/**
	 * make sure that the constraint we constructed the constraint panel is what is in the constraint panel
	 */
	@Test
	public void getConstraintTest() {
		assertEquals("ConstraintPanel should have 1 component",constraint,cp.getConstraint());
	}

	/**
	 * create a new constraint
	 * make sure that new constraint is not the same as the old one, and that the old one is what is in the constraint panel
	 * set the new constraint in the constraint panel
	 * make sure the new constraint is in the panel and the old one is not
	 */
	@Test
	public void setConstraintTest() {
		ForbidInstructorAssignmentConstraint c = new ForbidInstructorAssignmentConstraint();
		assertNotSame("ConstraintPanel should not be c",c,cp.getConstraint());
		assertEquals("ConstraintPanel should be constraint",constraint,cp.getConstraint());
		cp.setConstraint(c);
		assertEquals("ConstraintPanel should be c",c,cp.getConstraint());
		assertNotSame("ConstraintPanel should not be constraint",constraint,cp.getConstraint());
	}
}