package tlass.unitTesting.model;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.HashSet;

import junit.framework.TestCase;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import tlass.project.model.Assistant;
import tlass.project.model.Skill;
import tlass.project.model.TimeBlock;

public class TestAssistant {

	private Assistant assistant01;
	private Assistant assistant02;
	private Skill skill01, skill02;
	private TimeBlock time01, time02;
	private ArrayList<TimeBlock> timeList;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		assistant01 = new Assistant();
		assistant02 = new Assistant();
		skill01 = new Skill("skill01");
		skill02 = new Skill("skill02");
		time01 = new TimeBlock(0,0,0,0,0);
		time02 = new TimeBlock(1,1,1,1,1);
		timeList = new ArrayList<TimeBlock>();
		assistant01.setActive(true);
		assistant01.setName("Bob");
		assistant01.setStudentsLimit(40);
		assistant01.setUsername("bobo");
		assistant01.addSkill(skill01);
		assistant01.addUnavailableTime(time01);
		
		assistant02.setActive(false);
		assistant02.setName("Jill");
		assistant02.setStudentsLimit(20);
		assistant02.setUsername("jilly");
		assistant02.addSkill(skill02);
		assistant02.addUnavailableTime(time02);
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		assistant01 = null;
		assistant02 = null;
		skill01 = null;
		skill02 = null;
		time01 = null;
		time02 = null;
		timeList.clear();
	}

	/**
	 * tests the assistant constructor
	 */
	@Test
	public void testAssistant() {
		assertNotNull("Assistant02 constructor failed", assistant01);
		assertNotNull("Assistant01 constructor failed", assistant02);
	}
	
	/**
	 * tests the assistant copy method
	 */
	@Test
	public void testAssistantCopy() {
		assistant02 = assistant01.copy();
		assertEquals("Assistant copy failed for name", assistant01.getName(), assistant02.getName());
		assertEquals("Assistant copy failed for username", assistant01.getUsername(), assistant02.getUsername());
		assertEquals("Assistant copy failed for active field", assistant01.isActive(), assistant02.isActive());
		assertEquals("Assistant copy failed for student limit", assistant01.getStudentsLimit(), assistant02.getStudentsLimit());
		assertTrue("Assistant copy failed for skills", assistant01.getSkills().equals(assistant02.getSkills()));
		assertTrue("Assistant copy failed for unavailable times", 
				assistant01.getUnavailableTimes().equals(assistant02.getUnavailableTimes()));
	}
	
	/**
	 * Tests the assistant restore method
	 */
	@Test
	public void testRestore(){
		
		//Restore to the backup data
		assistant02.restore(assistant01);
		
		//assertEquals("")
		assertEquals("Assistant restore failed for name", assistant01.getName(), assistant02.getName());
		assertEquals("Assistant restore failed for username", assistant01.getUsername(), assistant02.getUsername());
		assertEquals("Assistant restore failed for active field", assistant01.isActive(), assistant02.isActive());
		assertEquals("Assistant restore failed for student limit", assistant01.getStudentsLimit(), assistant02.getStudentsLimit());
		assertTrue("Assistant restore failed for skills", assistant01.getSkills().equals(assistant02.getSkills()));
		assertTrue("Assistant restore failed for unavailable times", 
				assistant01.getUnavailableTimes().equals(assistant02.getUnavailableTimes()));
		
	}

	/**
	 * Tests the assistant toString method
	 */
	@Test
	public void testToString(){
		assertEquals("toString failed for assistant01", assistant01.toString(), "Bob");
		assertEquals("toString failed for assistant02", assistant02.toString(), "Jill");
	}
	
	/**
	 * Tests the assistant unavailable time method
	 */
	@Test
	public void testUnavailableTime(){
		
		//this one already exists, it should return false
		assertFalse("the unavailable time has been entered", assistant01.addUnavailableTime(time01));		
		//should return true meaning it does not exist in the list yet, and add it
		assertTrue("the unavailable time already exists", assistant01.addUnavailableTime(time02));
	}
	
	/**
	 * Tests the assistant addSkill method
	 */
	@Test
	public void testAddSkill(){
		assertTrue("addSkill failed for assistant01", assistant01.getSkills().contains(skill01));
		assertTrue("addSkill failed for assistant02", assistant02.getSkills().contains(skill02));
	}
	
	/**
	 * Tests the assistant unavailableFor method
	 */
	@Test
	public void testUnavailableFor(){
		assertTrue("assitant 01 is available at this time", assistant01.unavailableFor(time01));
		assertFalse("assitant 01 is not available at this time", assistant01.unavailableFor(time02));
	}
	
	/**
	 * Tests the assistant removeUnavailableTime method
	 */
	@Test
	public void testRemoveUnavailableTime(){
		assistant01.removeUnavailableTime(time01);
		assistant02.removeUnavailableTime(time02);
		assertFalse("remove time failed on assistant01", assistant01.getUnavailableTimes().contains(time01));
		assertFalse("remove time failed on assistant02", assistant02.getUnavailableTimes().contains(time02));
	}
	
	/**
	 * Tests the assistant removeSkill method
	 */
	@Test
	public void testRemoveSkill(){
		assistant01.removeSkill(skill01);
		assistant02.removeSkill(skill02);
		assertFalse("remove skill failed on assistant01", assistant01.getSkills().contains(skill01));
		assertFalse("remove skill failed on assistant02", assistant02.getSkills().contains(skill02));
	}
	
	/**
	 * Tests the assistant hasSkill method
	 */
	@Test
	public void testHasSkill(){
		assertTrue("assistant01 does not have skill01", assistant01.getSkills().contains(skill01));
		assertTrue("assistant02 does not have skill02", assistant02.getSkills().contains(skill02));
		assertFalse("assistant01 has skill02", assistant01.getSkills().contains(skill02));
	}
	
	/**
	 * Tests the assistant unavailableFor method for a LIST of times instead of just one
	 */
	@Test
	public void testUnavailableForTimeList(){
		//now assistant one is unavailable for both time01 and time02
		assistant01.addUnavailableTime(time02);
		//put time01 and time02 into an arraylist of timeblocks
		timeList.add(time01);
		timeList.add(time02);
		assertTrue("assistant 01 is available at these times", assistant01.unavailableFor(timeList));
		//remove the only time that was listed under unavailable times for assistant02
		assistant02.removeUnavailableTime(time02);
		assertFalse("assistant 02 is not available at this time", assistant02.unavailableFor(timeList));
	}
}