package tlass.unitTesting.model;

import static org.junit.Assert.*;

import java.util.ArrayList;

import junit.framework.TestCase;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import tlass.project.model.Assistant;
import tlass.project.model.Instructor;
/**
 * 
 * @author AndMac
 *
 */
public class TestInstructor {
	private Instructor instructor01;
	private Instructor instructor02;
	private ArrayList<Assistant> totalAssistants;
	private Assistant assistant01, assistant02;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		
	}

	/**
	 * Creates data for each test
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		instructor01 = new Instructor();
		instructor02 = new Instructor();
		totalAssistants = new ArrayList<Assistant>();
		assistant01 = new Assistant();
		assistant02 = new Assistant();
		
		instructor01.setName("Dr. Horrible");
		instructor01.setId("1234");
		assistant01.setName("Bob");
		totalAssistants.add(assistant01);
		instructor01.addForbiddenAssistant(assistant01);
		
		instructor02.setName("Dr. Dre");
		instructor02.setId("5678");
		assistant02.setName("Jill");
		totalAssistants.add(assistant02);
		instructor02.addForbiddenAssistant(assistant02);
	}

	/**
	 * Clears data after testing is complete
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		instructor01 = null;
		instructor02 = null;
		totalAssistants.clear();
		assistant01 = null;
		assistant02 = null;
	}

	/**
	 * Tests instructor creation
	 */
	@Test
	public void testInstructorCreation() {
		assertNotNull("Instructor constructor failed", instructor01);
		assertNotNull("Instructor constructor failed", instructor02);
	}
	/**
	 * Tests the instructor clone method
	 */
	@Test
	public void testInstructorClone(){
		instructor02 = instructor01.copy();
		assertEquals("Clone method failed for id", instructor01.getId(), instructor02.getId());
		assertEquals("Clone method failed for name", instructor01.getName(), instructor02.getName());
		assertTrue("Clone method failed for forbidden assistants", 
				instructor01.getForbiddenAssistants().equals(instructor02.getForbiddenAssistants()));
	}

	/**
	 * Tests the instructor clear method
	 */
	@Test
	public void testClearAssistants(){
		totalAssistants.remove(assistant02); //removes Jill
		//Jill should be removed from instructor02's forbidden assisntant list since she has been deleted from the master list
		instructor02.clean(totalAssistants);
		assertFalse("Assistants have not been successfully cleaned", instructor02.isForbiddenAssistant(assistant02));
	}
	
	/**
	 * Tests the instructor restore method
	 */
	@Test
	public void testRestoreInstructor(){
		
		//Restore to the backup data
		instructor02.restore(instructor01);
		
		assertEquals("Restore failed for name", instructor02.getName(), "Dr. Horrible");
		assertEquals("Restore failed for id", instructor02.getId(), "1234");
		assertTrue("Restore failed for assistant", instructor02.isForbiddenAssistant(assistant01));
		
	}
	
	/**
	 * Tests the instructor allow assistant method
	 */
	@Test
	public void testAllowAssistant(){
		
		instructor01.allowAssistant(assistant01);
		assertFalse("Allow assistant failed", instructor01.isForbiddenAssistant(assistant01));
		
	}
	
	/**
	 * Tests the instructor toString method
	 */
	@Test
	public void testInstructorToString(){
		
		assertEquals("toString failed", instructor01.toString(), "Dr. Horrible");
		
	}
	
}








