package tlass.unitTesting.model;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import junit.framework.TestCase;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import tlass.project.model.Assistant;
import tlass.project.model.Instructor;
import tlass.project.model.Position;
import tlass.project.model.Section;
import tlass.project.model.Skill;
import tlass.project.model.TimeBlock;

public class TestSection {

	private Section section, section2;
	private TimeBlock time;
	private HashSet<Skill> skillsNeeded;
	private ArrayList<Position> tas;
	private ArrayList<Position> las;
	private Instructor instructor;
	private Assistant assistant;
	private Position position;
	private HashMap<Position,Assistant> requiredAssistants;
	private HashMap<Position,ArrayList<Assistant>> forbiddenAssistants;
	private Skill skill;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		section = new Section();
		section2 = new Section();
		time = new TimeBlock(1,1,1,1,1);
		skillsNeeded = new HashSet<Skill>();
		tas = new ArrayList<Position>();
		las = new ArrayList<Position>();
		instructor = new Instructor();
		assistant = new Assistant();
		position = new Position(section, true, 1);
		skill = new Skill("A skill");
		requiredAssistants = new HashMap<Position,Assistant>();
		forbiddenAssistants = new HashMap<Position,ArrayList<Assistant>>();
		
		instructor.setName("Dr. Dre");
		
		section.setId("section01");
		section.addTime(time);
		section.addSkill(skill);
		section.setInstructor(instructor);
		
		//create section 2 with the same time so we can test for time conflict
		section2.addTime(time);
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		section = null;
		section2 = null;
		time = null;
		skillsNeeded.clear();
		tas.clear();
		las.clear();
		instructor = null;
		assistant = null;
		position = null;
		skill = null;
		requiredAssistants.clear();
		forbiddenAssistants.clear();
	}

	/**
	 * tests the section constructor
	 */
	@Test
	public void sectionTest() {
		assertNotNull("section constructor failed", section);
	}
	
	/**
	 * Tests the section toString method
	 */
	@Test
	public void toStringTest(){
		assertEquals("toString failed for section", section.toString(), "section01");
	}
	
	/**
	 * Tests the section addRequiredAssistant method
	 */
	@Test
	public void addRequiredAssistantTest(){
		section.addRequiredAssistant(position, assistant);
		assertTrue("addRequiredAssistant failed", section.getRequiredAssistants().containsValue(assistant));
	}
	
	/**
	 * Tests the section addTime method
	 */
	@Test
	public void addTimeTest(){
		assertTrue("addTime failed", section.getTime().contains(time));
	}

	/**
	 * Tests the section removeTime method
	 */
	@Test
	public void removeTimeTest(){
		section.removeTime(time);
		assertFalse("removeTime failed", section.getTime().contains(time));
	}
	
	/**
	 * Tests the section addLAPosition method
	 */
	@Test
	public void addLAPositionTest(){
		position = section.addLAPosition();
		assertTrue("LA position was not added", section.getLAs().contains(position));
	}
	
	/**
	 * Tests the section addTAPosition method
	 */
	@Test
	public void addTAPositionTest(){
		position = section.addTAPosition();
		assertTrue("TA position was not added", section.getTAs().contains(position));
	}
	
	/**
	 * Tests the section addSkill method
	 */
	@Test
	public void testAddSkill(){
		assertTrue("skill was not successfully added", section.getSkillsNeeded().contains(skill));
	}
	
	/**
	 * Tests the section removeSkill method
	 */
	@Test
	public void testRemoveSkill(){
		section.removeSkill(skill);
		assertFalse("skill was not successfully removed", section.getSkillsNeeded().contains(skill));
	}
	
	/**
	 * Tests the section intersect method
	 */
	@Test
	public void testIntersect(){
		assertTrue("section times do not intersect", section.intersect(section2));
	}
	
}