package tlass.unitTesting.project;

import static org.junit.Assert.*;

import java.sql.SQLException;
import java.util.ArrayList;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.internal.ArrayComparisonFailure;

import tlass.project.Controller;
import tlass.project.WebformHandler;
import tlass.project.exceptions.DbConnectionException;
import tlass.project.model.Schedule;

public class TestWebformHandler {
	private static Controller c;
	private static String s;
	private WebformHandler wh;

	/**
	 * create a controller for the class
	 * 
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		c = new Controller();
		s = "New Semester";
	}

	/**
	 * null the controller after the class
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		c = null;
	}

	/**
	 * create the webform handler for each test
	 * 
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		wh = new WebformHandler(c);
	}

	/**
	 * null the webform handler for each test
	 * 
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		wh = null;
	}

	@Test
	public void getAssignmentsTest() {
		ArrayList<Object> al = new ArrayList<Object>();
		try {
			assertEquals("The semester should come back with the assignments on the database",al,wh.getAssignments(s));
		} catch (DbConnectionException e) {
			fail("Database Connection Exception");
		} catch (SQLException e) {
			fail("SQL Exception");
		}
	}
	
	@Ignore
	@Test
	public void notifyAssistantTest() {
		fail("Not yet implemented");
	}
	
	@Test
	public void publishScheduleTest() {
		Schedule schedule = new Schedule();
		try {
			assertArrayEquals("Published schedule should return the given users",new String[]{},wh.publishSchedule(s,schedule));
		} catch (ArrayComparisonFailure e) {
			fail("Arrays did not compare right");
		} catch (DbConnectionException e) {
			fail("Database connection error");
		} catch (SQLException e) {
			fail("SQL exception");
		}
	}
	
	@Test
	public void finalizeScheduleTest() {
		Schedule schedule = new Schedule();
		try {
			assertArrayEquals("Finalized schedule should return the given users",new String[]{},wh.finalizeSchedule(s,schedule));
		} catch (ArrayComparisonFailure e) {
			fail("Arrays did not compare right");
		} catch (DbConnectionException e) {
			fail("Database connection error");
		} catch (SQLException e) {
			fail("SQL exception");
		}
	}
	
	@Test
	public void getNewResponsesTest() {
		Schedule schedule = new Schedule();
		ArrayList<Object> al = new ArrayList<Object>();
		try {
			assertEquals("Finalized schedule should return the given users",al,wh.getNewResponses(s,schedule));
		} catch (ArrayComparisonFailure e) {
			fail("Arrays did not compare right");
		} catch (DbConnectionException e) {
			fail("Database connection error");
		} catch (SQLException e) {
			fail("SQL exception");
		}
	}
	
	@Test
	public void getUnavailableAssistantsTest() {
		ArrayList<Object> al = new ArrayList<Object>();
		try {
			assertEquals("Finalized schedule should return the given users",al,wh.getUnavailableAssistants(s));
		} catch (ArrayComparisonFailure e) {
			fail("Arrays did not compare right");
		} catch (DbConnectionException e) {
			fail("Database connection error");
		} catch (SQLException e) {
			fail("SQL exception");
		}
	}
	
	@Test
	public void getUnavailableTimesTest() {
		ArrayList<Object> al = new ArrayList<Object>();
		try {
			assertEquals("Finalized schedule should return the given users",al,wh.getUnavailableTimes(s,"test"));
		} catch (ArrayComparisonFailure e) {
			fail("Arrays did not compare right");
		} catch (DbConnectionException e) {
			fail("Database connection error");
		} catch (SQLException e) {
			fail("SQL exception");
		}
	}
	
	@Test
	public void getAllUnavailableTimesTest() {
		ArrayList<Object> al = new ArrayList<Object>();
		try {
			assertEquals("Finalized schedule should return the given users",al,wh.getAllUnavailableTimes(s));
		} catch (ArrayComparisonFailure e) {
			fail("Arrays did not compare right");
		} catch (DbConnectionException e) {
			fail("Database connection error");
		} catch (SQLException e) {
			fail("SQL exception");
		}
	}
	
	@Test
	public void removeUnavailableTimesTest() {
		ArrayList<Object> al = new ArrayList<Object>();
		try {
			wh.removeUnavailableTimes(s);
			assertEquals("removing unavailable times should return the given users",al,wh.getAllUnavailableTimes(s));
		} catch (ArrayComparisonFailure e) {
			fail("Arrays did not compare right");
		} catch (DbConnectionException e) {
			fail("Database connection error");
		} catch (SQLException e) {
			fail("SQL exception");
		}
	}
}